@@ -1498,6 +1498,9 @@ f_eval_formula(f_val, f_str, v_str, pt_str) double *f_val;char *f_str;char *v_st
       }
       *f_val = FormulaVal;
       /* printf("\nEVAL formulaOK = (%.16g)\n",FormulaVal); */
+      if (isnan(FormulaVal)) {
+          return(1);
+      }
       return (0);
     }
   }
