@@ -105,7 +105,10 @@ foreach my $file ( glob("*.tex") ) {
 	if (!exists($fragmentLabels{$ref})
 	    && $ref ne 'course.manual.access.hlp'
 	    && $ref ne 'author.manual.access.hlp'
-            && $ref ne 'domain.manual.access.hlp') {
+            && $ref ne 'domain.manual.access.hlp'
+            && $ref ne 'course.manual.pdf'
+            && $ref ne 'author.manual.pdf'
+            && $ref ne 'domain.manual.pdf') {
 	    $error++;
 	    print("***ERROR: ref $ref in $file doesn't exist in label hash. \n");
 	} else {
