@@ -1,6 +1,6 @@
 # The LearningOnline Network
 #
-# $Id: Lond.pm,v 1.8 2015/05/22 21:14:59 raeburn Exp $
+# $Id: Lond.pm,v 1.8.2.2 2021/02/10 15:08:14 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -782,6 +782,53 @@ sub dump_profile_database {
     return $qresult;
 }
 
+sub is_course {
+    my ($cdom,$cnum) = @_;
+
+    return unless (($cdom =~ /^$LONCAPA::match_domain$/) &&
+                   ($cnum =~ /^$LONCAPA::match_courseid$/));
+    my $hashid = $cdom.':'.$cnum;
+    my ($iscourse,$cached) =
+        &Apache::lonnet::is_cached_new('iscourse',$hashid);
+    unless (defined($cached)) {
+        my $hashref =
+            &tie_domain_hash($cdom, "nohist_courseids", &GDBM_WRCREAT());
+        if (ref($hashref) eq 'HASH') {
+            my $esc_key = &escape($cdom.'_'.$cnum);
+            if (exists($hashref->{$esc_key})) {
+                $iscourse = 1;
+            } else {
+                $iscourse = 0;
+            }
+            &Apache::lonnet::do_cache_new('iscourse',$hashid,$iscourse,3600);
+            unless (&untie_domain_hash($hashref)) {
+                &logthis("Failed to untie tied hash for nohist_courseids.db for $cdom");
+            }
+        } else {
+            &logthis("Failed to tie hash for nohist_courseids.db for $cdom");
+        }
+    }
+    return $iscourse;
+}
+
+sub get_dom {
+    my ($userinput) = @_;
+    my ($cmd,$udom,$namespace,$what) =split(/:/,$userinput,4);
+    my $hashref = &tie_domain_hash($udom,$namespace,&GDBM_READER()) or
+        return "error: ".($!+0)." tie(GDBM) Failed while attempting $cmd";
+    my $qresult='';
+    if (ref($hashref)) {
+        chomp($what);
+        my @queries=split(/\&/,$what);
+        for (my $i=0;$i<=$#queries;$i++) {
+            $qresult.="$hashref->{$queries[$i]}&";
+        }
+        $qresult=~s/\&$//;
+    }
+    &untie_user_hash($hashref) or
+        return "error: ".($!+0)." untie(GDBM) Failed while attempting $cmd";
+    return $qresult;
+}
 
 1;
 
@@ -905,7 +952,18 @@ courseID -- for the course for which the data are being requested.
 The contents of the inner hash, for that single item in the outer hash
 are returned (and cached in memcache for 10 minutes).
 
+=item get_dom ( $userinput )
 
+get_dom() will retrieve domain configuration information from a GDBM file
+in /home/httpd/lonUsers/$dom on the primary library server in a domain.
+The single argument passed is the string: $cmd:$udom:$namespace:$what
+where $cmd is the command historically passed to lond - i.e., getdom
+or egetdom, $udom is the domain, $namespace is the name of the GDBM file
+(encconfig or configuration), and $what is a string containing names of
+items to retrieve from the db file (each item name is escaped and separated
+from the next item name with an ampersand). The return value is either:
+error: followed by an error message, or a string containing the value (escaped)
+for each item, again separated from the next item with an ampersand.
 
 =back
 
