@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Cookie Based Access Handler
 #
-# $Id: lonacc.pm,v 1.159.2.5 2017/01/21 20:40:46 raeburn Exp $
+# $Id: lonacc.pm,v 1.159.2.20 2021/02/10 11:36:15 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -159,7 +159,7 @@ sub get_posted_cgi {
                         if (length($value) == 1) {
                             $value=~s/[\r\n]$//;
                         }
-                    } elsif ($fname =~ /\.(xls|doc|ppt)x$/i) {
+                    } elsif ($fname =~ /\.(xls|doc|ppt)(x|m)$/i) {
                         $value=~s/[\r\n]$//;
                     }
                     if (ref($fields) eq 'ARRAY') {
@@ -281,7 +281,6 @@ sub upload_size_allowed {
 sub sso_login {
     my ($r,$handle,$username) = @_;
 
-    my $lonidsdir=$r->dir_config('lonIDsDir');
     if (($r->user eq '') || ($username ne '') || ($r->user eq 'public:public') ||
         (defined($env{'user.name'}) && (defined($env{'user.domain'}))
 	  && ($handle ne ''))) {
@@ -333,17 +332,21 @@ sub sso_login {
             ($is_balancer,$otherserver) =
                 &Apache::lonnet::check_loadbalancing($user,$domain,'login');
             if ($is_balancer) {
-                if ($otherserver eq '') {
+                # Check if browser sent a LON-CAPA load balancer cookie (and this is a balancer)
+                my ($found_server,$balancer_cookie) = &Apache::lonnet::check_for_balancer_cookie($r);
+                if (($found_server) && ($balancer_cookie =~ /^\Q$domain\E_\Q$user\E_/)) {
+                    $otherserver = $found_server;
+                } elsif ($otherserver eq '') {
                     my $lowest_load;
                     ($otherserver,undef,undef,undef,$lowest_load) = &Apache::lonnet::choose_server($domain);
                     if ($lowest_load > 100) {
                         $otherserver = &Apache::lonnet::spareserver($lowest_load,$lowest_load,1,$domain);
                     }
-                }
-                if ($otherserver ne '') {
-                    my @hosts = &Apache::lonnet::current_machine_ids();
-                    if (grep(/^\Q$otherserver\E$/,@hosts)) {
-                        $hosthere = $otherserver;
+                    if ($otherserver ne '') {
+                        my @hosts = &Apache::lonnet::current_machine_ids();
+                        if (grep(/^\Q$otherserver\E$/,@hosts)) {
+                            $hosthere = $otherserver;
+                        }
                     }
                 }
             }
@@ -366,7 +369,7 @@ sub sso_login {
 	} else {
 	    # need to login them in, so generate the need data that
 	    # migrate expects to do login
-            my $ip = $r->get_remote_host();
+            my $ip = &Apache::lonnet::get_requestor_ip($r);
 	    my %info=('ip'        => $ip,
 		      'domain'    => $domain,
 		      'username'  => $user,
@@ -498,7 +501,7 @@ sub handler {
                 my $preserved;
                 foreach my $pair (split(/&/,$query)) {
                     my ($name, $value) = split(/=/,$pair);
-                    unless ($name eq 'symb') {
+                    unless (($name eq 'symb') || ($name eq 'usehttp')) {
                         $preserved .= $pair.'&';
                     }
                     if (($env{'request.course.id'}) && ($name eq 'folderpath')) {
@@ -526,6 +529,7 @@ sub handler {
                         if ($value =~ /^supplemental/) {
                             $suppext = 1;
                         }
+                        last;
                     }
                 }
             }
@@ -553,34 +557,116 @@ sub handler {
         my $checkexempt;
         if ($env{'user.loadbalexempt'} eq $r->dir_config('lonHostID')) {
             if ($env{'user.loadbalcheck.time'} + 600 > time) {
-                $checkexempt = 1;    
+                $checkexempt = 1;
             }
         }
         if ($env{'user.noloadbalance'} eq $r->dir_config('lonHostID')) {
             $checkexempt = 1;
         }
-        unless ($checkexempt) {
+        unless (($checkexempt) || (($requrl eq '/adm/switchserver') && (!$r->is_initial_req()))) {
             ($is_balancer,$otherserver) =
                 &Apache::lonnet::check_loadbalancing($env{'user.name'},
                                                      $env{'user.domain'});
-        }
-        if ($is_balancer) {
-            $r->set_handlers('PerlResponseHandler'=>
-                             [\&Apache::switchserver::handler]);
-            if ($otherserver ne '') {
-                $env{'form.otherserver'} = $otherserver;
-            }
-            unless (($env{'form.origurl'}) || ($r->uri eq '/adm/roles') ||
-                    ($r->uri eq '/adm/switchserver') || ($r->uri eq '/adm/sso')) {
-                $env{'form.origurl'} = $r->uri;
+            if ($is_balancer) {
+                # Check if browser sent a LON-CAPA load balancer cookie (and this is a balancer)
+                my ($found_server,$balancer_cookie) = &Apache::lonnet::check_for_balancer_cookie($r);
+                if (($found_server) && ($balancer_cookie =~ /^\Q$env{'user.domain'}\E_\Q$env{'user.name'}\E_/)) {
+                    $otherserver = $found_server;
+                }
+                unless ($requrl eq '/adm/switchserver') {
+                    $r->set_handlers('PerlResponseHandler'=>
+                                     [\&Apache::switchserver::handler]);
+                }
+                if ($otherserver ne '') {
+                    $env{'form.otherserver'} = $otherserver;
+                }
+                unless (($env{'form.origurl'}) || ($r->uri eq '/adm/roles') ||
+                        ($r->uri eq '/adm/switchserver') || ($r->uri eq '/adm/sso')) {
+                    $env{'form.origurl'} = $r->uri;
+                }
             }
         }
 
 # ---------------------------------------------------------------- Check access
 	my $now = time;
-	if ($requrl !~ m{^/(?:adm|public|prtspool)/}
+        my ($check_symb,$check_access,$check_block,$access,$poss_symb);
+	if ($requrl !~ m{^/(?:adm|public|(?:prt|zip)spool)/}
 	    || $requrl =~ /^\/adm\/.*\/(smppg|bulletinboard)(\?|$ )/x) {
-	    my $access=&Apache::lonnet::allowed('bre',$requrl);
+            $check_access = 1;
+        }
+        if ((!$check_access) && ($env{'request.course.id'})) {
+            if (($requrl eq '/adm/viewclasslist') ||
+                ($requrl =~ m{^(/adm/wrapper|)\Q/uploaded/$cdom/$cnum/docs/\E}) ||
+                ($requrl =~ m{^/adm/.*/aboutme$}) ||
+                ($requrl=~m{^/adm/coursedocs/showdoc/})) {
+                $check_block = 1;
+            }
+        }
+        if (($env{'request.course.id'}) && (!$suppext)) {
+            $requrl=~/\.(\w+)$/;
+            if ((&Apache::loncommon::fileembstyle($1) eq 'ssi') ||
+                ($requrl=~/^\/adm\/.*\/(aboutme|smppg|bulletinboard)(\?|$ )/x) ||
+                ($requrl=~/^\/adm\/wrapper\//) ||
+                ($requrl=~m|^/adm/coursedocs/showdoc/|) ||
+                ($requrl=~m|\.problem/smpedit$|) ||
+                ($requrl=~/^\/public\/.*\/syllabus$/) ||
+                ($requrl=~/^\/adm\/(viewclasslist|navmaps)$/) ||
+                ($requrl=~/^\/adm\/.*\/aboutme\/portfolio(\?|$)/)) {
+                $check_symb = 1;
+            }
+        }
+        if (($check_access) || ($check_block)) {
+            if ($check_symb) {
+                if ($env{'form.symb'}) {
+                    $poss_symb=&Apache::lonnet::symbclean($env{'form.symb'});
+                } elsif (($env{'request.course.id'}) && ($r->args ne '')) {
+                    my $query = $r->args;
+                    foreach my $pair (split(/&/,$query)) {
+                        my ($name, $value) = split(/=/,$pair);
+                        $name = &unescape($name);
+                        $value =~ tr/+/ /;
+                        $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
+                        if ($name eq 'symb') {
+                            $poss_symb = &Apache::lonnet::symbclean($value);
+                            last;
+                        }
+                    }
+                }
+                if ($poss_symb) {
+                    my ($possmap,$resid,$url)=&Apache::lonnet::decode_symb($poss_symb);
+                    $url = &Apache::lonnet::clutter($url);
+                    my $toplevelmap = $env{'course.'.$env{'request.course.id'}.'.url'};
+                    unless (($url eq $requrl) && (($possmap eq $toplevelmap) ||
+                                                  (&Apache::lonnet::is_on_map($possmap)))) {
+                        undef($poss_symb);
+                    }
+                    if ($poss_symb) {
+                        if ((!$env{'request.role.adv'}) && ($env{'acc.randomout'}) &&
+                            ($env{'acc.randomout'}=~/\&\Q$poss_symb\E\&/)) {
+                            undef($poss_symb);
+                        }
+                    }
+                }
+                if ($poss_symb) {
+                    $access=&Apache::lonnet::allowed('bre',$requrl,$poss_symb);
+                } else {
+                    $access=&Apache::lonnet::allowed('bre',$requrl,'','','','',1);
+                }
+            } else {
+                $access=&Apache::lonnet::allowed('bre',$requrl);
+            }
+        }
+        if ($check_block) {
+            if ($access eq 'B') {
+                if ($poss_symb) {
+                    if (&Apache::lonnet::symbverify($poss_symb,$requrl)) {
+                        $env{'request.symb'} = $poss_symb;
+                    }
+                }
+                &Apache::blockedaccess::setup_handler($r);
+                return OK;
+            }
+        } elsif ($check_access) { 
             if ($handle eq '') {
                 unless ($access eq 'F') {
                     if ($requrl =~ m{^/res/$match_domain/$match_username/}) {
@@ -597,6 +683,11 @@ sub handler {
 		return OK;
 	    }
             if ($access eq 'B') {
+                if ($poss_symb) {
+                    if (&Apache::lonnet::symbverify($poss_symb,$requrl)) {
+                        $env{'request.symb'} = $poss_symb;
+                    }
+                }
                 &Apache::blockedaccess::setup_handler($r);
                 return OK;
             }
@@ -616,7 +707,7 @@ sub handler {
                         }
                     }
                 } elsif (($handle =~ /^publicuser_\d+$/) && (&Apache::lonnet::is_portfolio_url($requrl))) {
-                    my $clientip = $r->get_remote_host();
+                    my $clientip = &Apache::lonnet::get_requestor_ip($r);
                     if (&Apache::lonnet::allowed('bre',$requrl,undef,undef,$clientip) ne 'F') {
                         $env{'user.error.msg'}="$requrl:bre:1:1:Access Denied";
                         return HTTP_NOT_ACCEPTABLE;
@@ -647,7 +738,7 @@ sub handler {
 	    $env{'user.domain'} eq 'public' &&
 	    $requrl !~ m{^/+(res|public|uploaded)/} &&
 	    $requrl !~ m{^/adm/[^/]+/[^/]+/aboutme/portfolio$ }x &&
-        $requrl !~ m{^/adm/blockingstatus/.*$} &&
+            $requrl !~ m{^/adm/blockingstatus/.*$} &&
 	    $requrl !~ m{^/+adm/(help|logout|restrictedaccess|randomlabel\.png)}) {
 	    $env{'request.querystring'}=$r->args;
 	    $env{'request.firsturl'}=$requrl;
@@ -656,16 +747,8 @@ sub handler {
 # ------------------------------------------------------------- This is allowed
 	if ($env{'request.course.id'}) {
 	    &Apache::lonnet::countacc($requrl);
-	    $requrl=~/\.(\w+)$/;
             my $query=$r->args;
-	    if ((&Apache::loncommon::fileembstyle($1) eq 'ssi') ||
-		($requrl=~/^\/adm\/.*\/(aboutme|smppg|bulletinboard)(\?|$ )/x) ||
-		($requrl=~/^\/adm\/wrapper\//) ||
-		($requrl=~m|^/adm/coursedocs/showdoc/|) ||
-		($requrl=~m|\.problem/smpedit$|) ||
-		($requrl=~/^\/public\/.*\/syllabus$/) ||
-                ($requrl=~/^\/adm\/(viewclasslist|navmaps)$/) ||
-                ($requrl=~/^\/adm\/.*\/aboutme\/portfolio(\?|$)/)) {
+            if ($check_symb) {
 # ------------------------------------- This is serious stuff, get symb and log
 		my $symb;
 		if ($query) {
@@ -673,17 +756,42 @@ sub handler {
 		}
 		if ($env{'form.symb'}) {
 		    $symb=&Apache::lonnet::symbclean($env{'form.symb'});
-		    if ($requrl =~ m|^/adm/wrapper/|
-			|| $requrl =~ m|^/adm/coursedocs/showdoc/|) {
-			my ($map,$mid,$murl)=&Apache::lonnet::decode_symb($symb);
-			&Apache::lonnet::symblist($map,$murl => [$murl,$mid],
-						  'last_known' =>[$murl,$mid]);
+                    if (($requrl eq '/adm/navmaps') ||
+                        ($requrl =~ m{^/adm/wrapper/}) ||
+                        ($requrl =~ m{^/adm/coursedocs/showdoc/})) {
+                        unless (&Apache::lonnet::symbverify($symb,$requrl)) {
+                            if (&Apache::lonnet::is_on_map($requrl)) {
+                                $symb = &Apache::lonnet::symbread($requrl);
+                                unless (&Apache::lonnet::symbverify($symb,$requrl)) {
+                                    undef($symb);
+                                }
+                            }
+                        }
+                        if ($symb) {
+                            if ($requrl eq '/adm/navmaps') {
+                                my ($map,$mid,$murl)=&Apache::lonnet::decode_symb($symb);
+                                &Apache::lonnet::symblist($map,$murl => [$murl,$mid]);
+                            } elsif (($requrl =~ m{^/adm/wrapper/}) ||
+                                     ($requrl =~ m{^/adm/coursedocs/showdoc/})) {
+                                my ($map,$mid,$murl)=&Apache::lonnet::decode_symb($symb);
+                                if ($map =~ /\.page$/) {
+                                    my $mapsymb = &Apache::lonnet::symbread($map);
+                                    ($map,$mid,$murl)=&Apache::lonnet::decode_symb($mapsymb);
+                                }
+                                &Apache::lonnet::symblist($map,$murl => [$murl,$mid],
+                                                          'last_known' =>[$murl,$mid]);
+                            }
+                        }
 		    } elsif ((&Apache::lonnet::symbverify($symb,$requrl)) ||
 			     (($requrl=~m|(.*)/smpedit$|) &&
 			      &Apache::lonnet::symbverify($symb,$1)) ||
                              (($requrl=~m|(.*/aboutme)/portfolio$|) &&
                               &Apache::lonnet::symbverify($symb,$1))) {
 			my ($map,$mid,$murl)=&Apache::lonnet::decode_symb($symb);
+                        if (($map =~ /\.page$/) && ($requrl !~ /\.page$/)) {
+                            my $mapsymb = &Apache::lonnet::symbread($map);
+                            ($map,$mid,$murl)=&Apache::lonnet::decode_symb($mapsymb);
+                        }
 			&Apache::lonnet::symblist($map,$murl => [$murl,$mid],
 						  'last_known' =>[$murl,$mid]);
 		    } else {
@@ -697,24 +805,72 @@ sub handler {
                     if ($requrl=~m{^(/adm/.*/aboutme)/portfolio$}) {
                         $requrl = $1;
                     }
-                    unless ($suppext) {
-		        $symb=&Apache::lonnet::symbread($requrl);
-		        if (&Apache::lonnet::is_on_map($requrl) && $symb &&
-			    !&Apache::lonnet::symbverify($symb,$requrl)) {
-			    $r->log_reason('Invalid symb for '.$requrl.': '.$symb);
-			    $env{'user.error.msg'}=
-			        "$requrl:bre:1:1:Invalid Access";
-			    return HTTP_NOT_ACCEPTABLE; 
-		        }
-		        if ($symb) {
-			    my ($map,$mid,$murl)=
-			        &Apache::lonnet::decode_symb($symb);
-			    &Apache::lonnet::symblist($map,$murl =>[$murl,$mid],
-						      'last_known' =>[$murl,$mid]);
-		        }
+		    $symb=&Apache::lonnet::symbread($requrl);
+                    if (&Apache::lonnet::is_on_map($requrl) && $symb) {
+                        my ($encstate,$invalidsymb);
+                        unless (&Apache::lonnet::symbverify($symb,$requrl,\$encstate)) {
+                            $invalidsymb = 1;
+                            #
+                            # If $env{'request.enc'} inconsistent with encryption expected for $symb
+                            # retrieved by lonnet::symbread(), call again to check for an instance of
+                            # $requrl in the course for which expected encryption matches request.enc.
+                            # If symb for different instance passes lonnet::symbverify(), use that as
+                            # the symb for $requrl and call &Apache::lonnet::allowed() for that symb.
+                            # Report invalid symb if there is no other symb. Redirect to /adm/ambiguous
+                            # if multiple possible symbs consistent with request.enc available for $requrl.
+                            #
+                            if (($env{'request.enc'} && !$encstate) || (!$env{'request.enc'} && $encstate)) {
+                                my %possibles;
+                                my $nocache = 1;
+                                my $oldsymb = $symb;
+                                $symb = &Apache::lonnet::symbread($requrl,'','','',\%possibles,$nocache);
+                                if (($symb) && ($symb ne $oldsymb)) {
+                                    if (&Apache::lonnet::symbverify($symb,$requrl)) {
+                                        my $access=&Apache::lonnet::allowed('bre',$requrl,$symb);
+                                        if ($access eq 'B') {
+                                            $env{'request.symb'} = $symb;
+                                            &Apache::blockedaccess::setup_handler($r);
+                                            return OK;
+                                        } elsif (($access eq '2') || ($access eq 'F')) {
+                                            $invalidsymb = '';
+                                        }
+                                    }
+                                } elsif (keys(%possibles) > 1) {
+                                    $r->internal_redirect('/adm/ambiguous');
+                                    return OK;
+                                }
+                            }
+                            if ($invalidsymb) {
+                                if ($requrl eq '/adm/navmaps') {
+                                    undef($symb);
+                                } else {
+                                    $r->log_reason('Invalid symb for '.$requrl.': '.$symb);
+                                    $env{'user.error.msg'}=
+                                        "$requrl:bre:1:1:Invalid Access";
+                                    return HTTP_NOT_ACCEPTABLE;
+                                }
+                            }
+                        }
+                    }
+		    if ($symb) {
+			my ($map,$mid,$murl)=
+			    &Apache::lonnet::decode_symb($symb);
+                        if ($requrl eq '/adm/navmaps') {
+                            &Apache::lonnet::symblist($map,$murl =>[$murl,$mid]);
+                        } else {
+                            if (($map =~ /\.page$/) && ($requrl !~ /\.page$/)) {
+                                my $mapsymb = &Apache::lonnet::symbread($map);
+                                ($map,$mid,$murl)=&Apache::lonnet::decode_symb($mapsymb);
+                            }
+                            &Apache::lonnet::symblist($map,$murl =>[$murl,$mid],
+                                                      'last_known' =>[$murl,$mid]);
+                        }
 		    }
 		}
 		$env{'request.symb'}=$symb;
+                if (($env{'request.symbread.cached.'}) && ($env{'request.symbread.cached.'} ne $symb)) {
+                    $env{'request.symbread.cached.'} = $symb;
+                }
 		&Apache::lonnet::courseacclog($symb);
 	    } else {
 # ------------------------------------------------------- This is other content
@@ -759,7 +915,7 @@ sub handler {
     }
 # ------------------------------------ See if this is a viewable portfolio file
     if (&Apache::lonnet::is_portfolio_url($requrl)) {
-        my $clientip = $r->get_remote_host();
+        my $clientip = &Apache::lonnet::get_requestor_ip($r);
 	my $access=&Apache::lonnet::allowed('bre',$requrl,undef,undef,$clientip);
 	if ($access eq 'A') {
 	    &Apache::restrictedaccess::setup_handler($r);
