@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # User Authentication Module
 #
-# $Id: lonauth.pm,v 1.121.2.16 2017/06/06 22:34:37 raeburn Exp $
+# $Id: lonauth.pm,v 1.121.2.21 2021/01/04 03:48:29 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -32,8 +32,6 @@ use strict;
 use LONCAPA;
 use Apache::Constants qw(:common);
 use CGI qw(:standard);
-use DynaLoader; # for Crypt::DES version
-use Crypt::DES;
 use Apache::loncommon();
 use Apache::lonnet;
 use Apache::lonmenu();
@@ -42,11 +40,12 @@ use Fcntl qw(:flock);
 use Apache::lonlocal;
 use Apache::File();
 use HTML::Entities;
+use Digest::MD5;
  
 # ------------------------------------------------------------ Successful login
 sub success {
     my ($r, $username, $domain, $authhost, $lowerurl, $extra_env,
-	$form) = @_;
+	$form,$cid) = @_;
 
 # ------------------------------------------------------------ Get cookie ready
     my $cookie =
@@ -60,8 +59,9 @@ sub success {
 
 # -------------------------------------------------------------------- Log this
 
+    my $ip = &Apache::lonnet::get_requestor_ip();
     &Apache::lonnet::log($domain,$username,$authhost,
-                         "Login $ENV{'REMOTE_ADDR'}");
+                         "Login $ip");
 
 # ------------------------------------------------- Check for critical messages
 
@@ -72,8 +72,27 @@ sub success {
         }
     }
 
-# ------------------------------------------------------------ Get cookie ready
-    $cookie="lonID=$cookie; path=/; HttpOnly";
+# ------------------------------------------------------------ Get cookies ready
+    my ($securecookie,$defaultcookie);
+    my $ssl = $r->subprocess_env('https');
+    if ($ssl) {
+        $securecookie="lonSID=$cookie; path=/; HttpOnly; secure";
+        my $lonidsdir=$r->dir_config('lonIDsDir');
+        if (($lonidsdir) && (-e "$lonidsdir/$cookie.id")) {
+            my $linkname=substr(Digest::MD5::md5_hex(Digest::MD5::md5_hex(time(). {}. rand(). $$)), 0, 32).'_linked';
+            if (-e "$lonidsdir/$linkname.id") {
+                unlink("$lonidsdir/$linkname.id");
+            }
+            my $made_symlink = eval { symlink("$lonidsdir/$cookie.id",
+                                              "$lonidsdir/$linkname.id"); 1 };
+            if ($made_symlink) {
+                $defaultcookie = "lonLinkID=$linkname; path=/; HttpOnly;";
+                &Apache::lonnet::appenv({'user.linkedenv' => $linkname});
+            }
+        }
+    } else {
+        $defaultcookie = "lonID=$cookie; path=/; HttpOnly;";
+    }
 # -------------------------------------------------------- Menu script and info
     my $destination = $lowerurl;
 
@@ -99,16 +118,26 @@ sub success {
     }
     if (defined($form->{symb})) {
         my $destsymb = $form->{symb};
+        my $encrypted;
+        if ($destsymb =~ m{^/enc/}) {
+            $encrypted = 1;
+            if ($cid) {
+                $destsymb = &Apache::lonenc::unencrypted($destsymb,$cid);
+            }
+        }
         $destination  .= ($destination =~ /\?/) ? '&' : '?';
         if ($destsymb =~ /___/) {
-            # FIXME Need to deal with encrypted symbs and urls as needed.
             my ($map,$resid,$desturl)=split(/___/,$destsymb);
             $desturl = &Apache::lonnet::clutter($desturl);
+            if ($encrypted) {
+                $desturl = &Apache::lonenc::encrypted($desturl,1,$cid);
+                $destsymb = $form->{symb};
+            }
             $desturl = &HTML::Entities::encode($desturl,'"<>&');
             $destsymb = &HTML::Entities::encode($destsymb,'"<>&');
             $destination .= 'destinationurl='.$desturl.
                             '&destsymb='.$destsymb;
-        } else {
+        } elsif (!$encrypted) {
             $destsymb = &HTML::Entities::encode($destsymb,'"<>&');
             $destination .= 'destinationurl='.$destsymb;
         }
@@ -138,7 +167,12 @@ sub success {
 # ------------------------------------------------- Output for successful login
 
     &Apache::loncommon::content_type($r,'text/html');
-    $r->header_out('Set-cookie' => $cookie);
+    if ($securecookie) {
+        $r->headers_out->add('Set-cookie' => $securecookie);
+    }
+    if ($defaultcookie) {
+        $r->headers_out->add('Set-cookie' => $defaultcookie);
+    }
     $r->send_http_header;
 
     my %lt=&Apache::lonlocal::texthash(
@@ -301,10 +335,11 @@ sub handler {
 
 # split user logging in and "su"-user
 
-    ($form{'uname'},$form{'suname'})=split(/\:/,$form{'uname'});
+    ($form{'uname'},$form{'suname'},$form{'sudom'})=split(/\:/,$form{'uname'});
     $form{'uname'} = &LONCAPA::clean_username($form{'uname'});
     $form{'suname'}= &LONCAPA::clean_username($form{'suname'});
-    $form{'udom'}  = &LONCAPA::clean_domain(  $form{'udom'});
+    $form{'udom'}  = &LONCAPA::clean_domain($form{'udom'});
+    $form{'sudom'} = &LONCAPA::clean_domain($form{'sudom'});
 
     my $role   = $r->dir_config('lonRole');
     my $domain = $r->dir_config('lonDefDomain');
@@ -429,20 +464,68 @@ sub handler {
 
 # --------------------------------- Are we attempting to login as somebody else?
     if ($form{'suname'}) {
+        my ($suname,$sudom,$sudomref);
+        $suname = $form{'suname'};
+        $sudom = $form{'udom'};
+        if ($form{'sudom'}) {
+            unless ($sudom eq $form{'sudom'}) {
+                if (&Apache::lonnet::domain($form{'sudom'})) {
+                    $sudomref = [$form{'sudom'}];
+                    $sudom = $form{'sudom'};
+                }
+            }
+        }
 # ------------ see if the original user has enough privileges to pull this stunt
-	if (&Apache::lonnet::privileged($form{'uname'},$form{'udom'})) {
+	if (&Apache::lonnet::privileged($form{'uname'},$form{'udom'},$sudomref)) {
 # ---------------------------------------------------- see if the su-user exists
-	    unless (&Apache::lonnet::homeserver($form{'suname'},$form{'udom'})
-		eq 'no_host') {
-		&Apache::lonnet::logthis(&Apache::lonnet::homeserver($form{'suname'},$form{'udom'}));
+	    unless (&Apache::lonnet::homeserver($suname,$sudom) eq 'no_host') {
 # ------------------------------ see if the su-user is not too highly privileged
-		unless (&Apache::lonnet::privileged($form{'suname'},$form{'udom'})) {
+		if (&Apache::lonnet::privileged($suname,$sudom)) {
+                    &Apache::lonnet::logthis('Attempted switch user to privileged user');
+                } else {
+                    my $noprivswitch;
+#
+# su-user's home server and user's home server must have one of:
+# (a) same domain
+# (b) same primary library server for the two domains
+# (c) same "internet domain" for primary library server(s) for home servers' domains
+#
+                    my $suprim = &Apache::lonnet::domain($sudom,'primary');
+                    my $suintdom = &Apache::lonnet::internet_dom($suprim);
+                    unless ($sudom eq $form{'udom'}) {
+                        my $uprim = &Apache::lonnet::domain($form{'udom'},'primary');
+                        my $uintdom = &Apache::lonnet::internet_dom($uprim);
+                        unless ($suprim eq $uprim) {
+                            unless ($suintdom eq $uintdom) {
+                                &Apache::lonnet::logthis('Attempted switch user '
+                                   .'to user with different "internet domain".');
+                                $noprivswitch = 1;
+                            }
+                        }
+                    }
+
+                    unless ($noprivswitch) {
+#
+# server where log-in occurs must have same "internet domain" as su-user's home
+# server
+#
+                        my $lonhost = $r->dir_config('lonHostID');
+                        my $hostintdom = &Apache::lonnet::internet_dom($lonhost);
+                        if ($hostintdom ne $suintdom) {
+                            &Apache::lonnet::logthis('Attempted switch user on a '
+                                .'server with a different "internet domain".');
+                        } else {
+
 # -------------------------------------------------------- actually switch users
-		    &Apache::lonnet::logperm('User '.$form{'uname'}.' at '.$form{'udom'}.
-			' logging in as '.$form{'suname'});
-		    $form{'uname'}=$form{'suname'};
-		} else {
-		    &Apache::lonnet::logthis('Attempted switch user to privileged user');
+
+			    &Apache::lonnet::logperm('User '.$form{'uname'}.' at '.
+				$form{'udom'}.' logging in as '.$suname.':'.$sudom);
+			    $form{'uname'}=$suname;
+                            if ($form{'udom'} ne $sudom) {
+                                $form{'udom'}=$sudom;
+                            }
+                        }
+                    }
 		}
 	    }
 	} else {
@@ -456,6 +539,11 @@ sub handler {
         ($is_balancer,$otherserver) =
             &Apache::lonnet::check_loadbalancing($form{'uname'},$form{'udom'},'login');
         if ($is_balancer) {
+            # Check if browser sent a LON-CAPA load balancer cookie (and this is a balancer)
+            my ($found_server,$balancer_cookie) = &Apache::lonnet::check_for_balancer_cookie($r);
+            if (($found_server) && ($balancer_cookie =~ /^\Q$env{'user.domain'}\E_\Q$env{'user.name'}\E_/)) {
+                $otherserver = $found_server;
+            }
             if ($otherserver eq '') {
                 my $lowest_load;
                 ($otherserver,undef,undef,undef,$lowest_load) = &Apache::lonnet::choose_server($form{'udom'});
@@ -488,6 +576,8 @@ sub handler {
             }
             $r->internal_redirect($switchto);
         } else {
+            &Apache::loncommon::content_type($r,'text/html');
+            $r->send_http_header;
             $r->print(&noswitch());
         }
         return OK;
@@ -509,6 +599,8 @@ sub handler {
                 }
                 $r->internal_redirect($switchto);
             } else {
+                &Apache::loncommon::content_type($r,'text/html');
+                $r->send_http_header;
                 $r->print(&noswitch());
             }
             return OK;
