@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Login Screen
 #
-# $Id: lonlogin.pm,v 1.158.2.6 2016/08/13 20:11:58 raeburn Exp $
+# $Id: lonlogin.pm,v 1.158.2.12 2021/01/04 03:49:10 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -37,7 +37,8 @@ use Apache::lonauth();
 use Apache::lonlocal;
 use Apache::migrateuser();
 use lib '/home/httpd/lib/perl/';
-use LONCAPA;
+use LONCAPA qw(:DEFAULT :match);
+use CGI::Cookie();
  
 sub handler {
     my $r = shift;
@@ -46,7 +47,7 @@ sub handler {
 	(join('&',$ENV{'QUERY_STRING'},$env{'request.querystring'},
 	      $ENV{'REDIRECT_QUERY_STRING'}),
 	 ['interface','username','domain','firsturl','localpath','localres',
-	  'token','role','symb','iptoken']);
+	  'token','role','symb','iptoken','btoken']);
     if (!defined($env{'form.firsturl'})) {
         &Apache::lonacc::get_posted_cgi($r,['firsturl']);
     }
@@ -56,9 +57,39 @@ sub handler {
 	return &Apache::migrateuser::handler($r);
     }
 
+# For "public user" - remove any exising "public" cookie, as user really wants to log-in
+    my ($handle,$lonidsdir,$expirepub,$userdom);
+    $lonidsdir=$r->dir_config('lonIDsDir');
+    unless ($r->header_only) {
+        $handle = &Apache::lonnet::check_for_valid_session($r,'lonID',undef,\$userdom);
+        if ($handle ne '') {
+            if ($handle=~/^publicuser\_/) {
+                unlink($r->dir_config('lonIDsDir')."/$handle.id");
+                undef($handle);
+                undef($userdom);
+                $expirepub = 1;
+            }
+        }
+    }
+
     &Apache::loncommon::no_cache($r);
     &Apache::lonlocal::get_language_handle($r);
     &Apache::loncommon::content_type($r,'text/html');
+    if ($expirepub) {
+        my $c = new CGI::Cookie(-name    => 'lonPubID',
+                                -value   => '',
+                                -expires => '-10y',);
+        $r->header_out('Set-cookie' => $c);
+    } elsif (($handle eq '') && ($userdom ne '')) {
+        my %cookies=CGI::Cookie->parse($r->header_in('Cookie'));
+        foreach my $name (keys(%cookies)) {
+            next unless ($name =~ /^lon(|S|Link|Pub)ID$/);
+            my $c = new CGI::Cookie(-name    => $name,
+                                    -value   => '',
+                                    -expires => '-10y',);
+            $r->headers_out->add('Set-cookie' => $c);
+        }
+    }
     $r->send_http_header;
     return OK if $r->header_only;
 
@@ -70,35 +101,89 @@ sub handler {
 	return OK;
     }
 
+    my $lonhost = $r->dir_config('lonHostID');
+    $env{'form.firsturl'} =~ s/(`)/'/g;
+
+# Check if browser sent a LON-CAPA load balancer cookie (and this is a balancer)
+
+    my ($found_server,$balancer_cookie) = &Apache::lonnet::check_for_balancer_cookie($r,1);
+    if ($found_server) {
+        my $hostname = &Apache::lonnet::hostname($found_server);
+        if ($hostname ne '') {
+            my $protocol = $Apache::lonnet::protocol{$found_server};
+            $protocol = 'http' if ($protocol ne 'https');
+            my $dest = '/adm/roles';
+            if ($env{'form.firsturl'} ne '') {
+                $dest = $env{'form.firsturl'};
+            }
+            my %info = (
+                         balcookie => $lonhost.':'.$balancer_cookie,
+                       );
+            my $balancer_token = &Apache::lonnet::tmpput(\%info,$found_server);
+            if ($balancer_token) {
+                $dest .=  (($dest=~/\?/)?'&;':'?') . 'btoken='.$balancer_token;
+            }
+            my $url = $protocol.'://'.$hostname.$dest;
+            my $start_page =
+                &Apache::loncommon::start_page('Switching Server ...',undef,
+                                               {'redirect'       => [0,$url],});
+            my $end_page   = &Apache::loncommon::end_page();
+            $r->print($start_page.$end_page);
+            return OK;
+        }
+    }
+
+#
+# Check if a LON-CAPA load balancer sent user here because user's browser sent
+# it a balancer cookie for an active session on this server.
+#
+
+    my $balcookie;
+    if ($env{'form.btoken'}) {
+        my %info = &Apache::lonnet::tmpget($env{'form.btoken'});
+        $balcookie = $info{'balcookie'};
+        &Apache::lonnet::tmpdel($env{'form.btoken'});
+        delete($env{'form.btoken'});
+    }
+
+#
+# If browser sent an old cookie for which the session file had been removed
+# check if configuration for user's domain has a portal URL set.  If so
+# switch user's log-in to the portal.
+#
+
+    if (($handle eq '') && ($userdom ne '')) {
+        my %domdefaults = &Apache::lonnet::get_domain_defaults($userdom);
+        if ($domdefaults{'portal_def'} =~ /^https?\:/) {
+            my $start_page = &Apache::loncommon::start_page('Switching Server ...',undef,
+                                          {'redirect' => [0,$domdefaults{'portal_def'}],});
+            my $end_page   = &Apache::loncommon::end_page();
+            $r->print($start_page.$end_page);
+            return OK;
+        }
+    }
+
     $env{'form.firsturl'} =~ s/(`)/'/g;
 
 # -------------------------------- Prevent users from attempting to login twice
-    my $handle = &Apache::lonnet::check_for_valid_session($r);
     if ($handle ne '') {
-        my $lonidsdir=$r->dir_config('lonIDsDir');
-        if ($handle=~/^publicuser\_/) {
-# For "public user" - remove it, we apparently really want to login
-	    unlink($r->dir_config('lonIDsDir')."/$handle.id");
-        } else {
-# Indeed, a valid token is found
-            &Apache::lonnet::transfer_profile_to_env($lonidsdir,$handle);
-	    my $start_page = 
-	        &Apache::loncommon::start_page('Already logged in');
-	    my $end_page = 
-	        &Apache::loncommon::end_page();
-            my $dest = '/adm/roles';
-            if ($env{'form.firsturl'} ne '') {
-                $dest = $env{'form.firsturl'}; 
-            }
-	    $r->print(
-                  $start_page
-                 .'<p class="LC_warning">'.&mt('You are already logged in!').'</p>'
-                 .'<p>'.&mt('Please either [_1]continue the current session[_2] or [_3]log out[_4].',
-                  '<a href="'.$dest.'">','</a>','<a href="/adm/logout">','</a>').'</p>'
-                 .$end_page
-                 );
-            return OK;
+        &Apache::lonnet::transfer_profile_to_env($lonidsdir,$handle);
+	my $start_page = 
+	    &Apache::loncommon::start_page('Already logged in');
+	my $end_page = 
+	    &Apache::loncommon::end_page();
+        my $dest = '/adm/roles';
+        if ($env{'form.firsturl'} ne '') {
+            $dest = $env{'form.firsturl'}; 
         }
+	$r->print(
+              $start_page
+             .'<p class="LC_warning">'.&mt('You are already logged in!').'</p>'
+             .'<p>'.&mt('Please either [_1]continue the current session[_2] or [_3]log out[_4].',
+              '<a href="'.$dest.'">','</a>','<a href="/adm/logout">','</a>').'</p>'
+             .$end_page
+             );
+        return OK;
     }
 
 # ---------------------------------------------------- No valid token, continue
@@ -127,12 +212,11 @@ sub handler {
     my $iconpath= 
 	&Apache::loncommon::lonhttpdurl($r->dir_config('lonIconsURL'));
 
-    my $lonhost = $r->dir_config('lonHostID');
     my $domain = &Apache::lonnet::default_login_domain();
     my $defdom = $domain;
     if ($lonhost ne '') {
         unless ($sessiondata{'sessionserver'}) {
-            my $redirect = &check_loginvia($domain,$lonhost);
+            my $redirect = &check_loginvia($domain,$lonhost,$lonidsdir,$balcookie);
             if ($redirect) {
                 $r->print($redirect);
                 return OK;
@@ -141,7 +225,7 @@ sub handler {
     }
 
     if (($sessiondata{'domain'}) &&
-        (&Apache::lonnet::domain($env{'form.domain'},'description'))) {
+        (&Apache::lonnet::domain($sessiondata{'domain'},'description'))) {
         $domain=$sessiondata{'domain'};
     } elsif (($env{'form.domain'}) && 
 	(&Apache::lonnet::domain($env{'form.domain'},'description'))) {
@@ -244,22 +328,36 @@ sub handler {
             &Apache::lonnet::logthis('No valid logtoken for log-in page -- unable to determine hostname for hostID: '.$lonhost.'. Check entry in hosts.tab');
         }
         my $spares='';
-	my $last;
-        foreach my $hostid (sort
-			    {
-				&Apache::lonnet::hostname($a) cmp
-				    &Apache::lonnet::hostname($b);
-			    }
-			    keys(%Apache::lonnet::spareid)) {
+        my (@sparehosts,%spareservers);
+        my $sparesref = &Apache::lonnet::this_host_spares($defdom);
+        if (ref($sparesref) eq 'HASH') {
+            foreach my $key (keys(%{$sparesref})) {
+                if (ref($sparesref->{$key}) eq 'ARRAY') {
+                    my @sorted = sort { &Apache::lonnet::hostname($a) cmp
+                                        &Apache::lonnet::hostname($b);
+                                      } @{$sparesref->{$key}};
+                    if (@sorted) {
+                        if ($key eq 'primary') {
+                            unshift(@sparehosts,@sorted);
+                        } elsif ($key eq 'default') {
+                            push(@sparehosts,@sorted);
+                        }
+                    }
+                }
+            }
+        }
+        foreach my $hostid (@sparehosts) {
             next if ($hostid eq $lonhost);
 	    my $hostname = &Apache::lonnet::hostname($hostid);
-	    next if (($last eq $hostname) || ($hostname eq ''));
-            $spares.='<br /><font size="+1"><a href="http://'.
+	    next if (($hostname eq '') || ($spareservers{$hostname}));
+            $spareservers{$hostname} = 1;
+            my $protocol = $Apache::lonnet::protocol{$hostid};
+            $protocol = 'http' if ($protocol ne 'https');
+            $spares.='<br /><span style="font-size: larger;"><a href="'.$protocol.'://'.
                 $hostname.
                 '/adm/login?domain='.$authdomain.'">'.
                 $hostname.'</a>'.
-                ' '.&mt('(preferred)').'</font>'.$/;
-	    $last=$hostname;
+                ' '.&mt('(preferred)').'</span>'.$/;
         }
         if ($spares) {
             $spares.= '<br />';
@@ -271,23 +369,26 @@ sub handler {
 			    &Apache::lonnet::hostname($b);
 		    }
 		    keys(%all_hostnames)) {
-            next if ($hostid eq $lonhost || $Apache::lonnet::spareid{$hostid});
+            next if ($hostid eq $lonhost);
             my $hostname = &Apache::lonnet::hostname($hostid);
-            next if (($last eq $hostname) || ($hostname eq ''));
-            $spares.='<br /><a href="http://'.
+            next if (($hostname eq '') || ($spareservers{$hostname}));
+            $spareservers{$hostname} = 1;
+            my $protocol = $Apache::lonnet::protocol{$hostid};
+            $protocol = 'http' if ($protocol ne 'https');
+            $spares.='<br /><a href="'.$protocol.'://'.
 	             $hostname.
 	             '/adm/login?domain='.$authdomain.'">'.
 	             $hostname.'</a>';
-            $last=$hostname;
          }
          $r->print(
-   '<html>'
-  .'<head><title>'
+   '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'
+  .'<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">'
+  .'<head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" /><title>'
   .&mt('The LearningOnline Network with CAPA')
   .'</title></head>'
   .'<body bgcolor="#FFFFFF">'
   .'<h1>'.&mt('The LearningOnline Network with CAPA').'</h1>'
-  .'<img src="/adm/lonKaputt/lonlogo_broken.gif" align="right" />'
+  .'<img src="/adm/lonKaputt/lonlogo_broken.gif" alt="broken icon" align="right" />'
   .'<h3>'.&mt('This LON-CAPA server is temporarily not available for login.').'</h3>');
         if ($spares) {
             $r->print('<p>'.&mt('Please attempt to login to one of the following servers:')
@@ -402,7 +503,7 @@ ENDSCRIPT
         my $omitextra;
         if ($headextra_exempt ne '') {
             my @exempt = split(',',$headextra_exempt);
-            my $ip = $ENV{'REMOTE_ADDR'};
+            my $ip = &Apache::lonnet::get_requestor_ip();
             if (grep(/^\Q$ip\E$/,@exempt)) {
                 $omitextra = 1;
             }
@@ -635,8 +736,8 @@ ENDDOCUMENT
 }
 
 sub check_loginvia {
-    my ($domain,$lonhost) = @_;
-    if ($domain eq '' || $lonhost eq '') {
+    my ($domain,$lonhost,$lonidsdir,$balcookie) = @_;
+    if ($domain eq '' || $lonhost eq '' || $lonidsdir eq '') {
         return;
     }
     my %domconfhash = &Apache::loncommon::get_domainconf($domain);
@@ -645,7 +746,7 @@ sub check_loginvia {
     my $output;
     if ($loginvia ne '') {
         my $noredirect;
-        my $ip = $ENV{'REMOTE_ADDR'};
+        my $ip = &Apache::lonnet::get_requestor_ip();
         if ($ip eq '127.0.0.1') {
             $noredirect = 1;
         } else {
@@ -665,6 +766,36 @@ sub check_loginvia {
             }
             if ($newhost ne $lonhost) {
                 if (&Apache::lonnet::hostname($newhost) ne '') {
+                    if ($balcookie) {
+                        my ($balancer,$cookie) = split(/:/,$balcookie);
+                        if ($cookie =~ /^($match_domain)_($match_username)_([a-f0-9]+)$/) {
+                            my ($udom,$uname,$cookieid) = ($1,$2,$3);
+                            unless (&Apache::lonnet::delbalcookie($cookie,$balancer) eq 'ok') {
+                                if ((-d $lonidsdir) && (opendir(my $dh,$lonidsdir))) {
+                                    while (my $filename=readdir($dh)) {
+                                        if ($filename=~/^(\Q$uname\E_\d+_\Q$udom\E_$match_lonid)\.id$/) {
+                                            my $handle = $1;
+                                            my %hash =
+                                                &Apache::lonnet::get_sessionfile_vars($handle,$lonidsdir,
+                                                                                     ['request.balancercookie',
+                                                                                      'user.linkedenv']);
+                                            if ($hash{'request.balancercookie'} eq "$balancer:$cookieid") {
+                                                if (unlink("$lonidsdir/$filename")) {
+                                                    if (($hash{'user.linkedenv'} =~ /^[a-f0-9]+_linked$/) &&
+                                                        (-l "$lonidsdir/$hash{'user.linkedenv'}.id") &&
+                                                        (readlink("$lonidsdir/$hash{'user.linkedenv'}.id") eq "$lonidsdir/$filename")) {
+                                                        unlink("$lonidsdir/$hash{'user.linkedenv'}.id");
+                                                    }
+                                                }
+                                            }
+                                            last;
+                                        }
+                                    }
+                                    closedir($dh);
+                                }
+                            }
+                        }
+                    }
                     $output = &redirect_page($newhost,$path);
                 }
             }
@@ -675,12 +806,13 @@ sub check_loginvia {
 
 sub redirect_page {
     my ($desthost,$path) = @_;
+    my $hostname = &Apache::lonnet::hostname($desthost);
     my $protocol = $Apache::lonnet::protocol{$desthost};
     $protocol = 'http' if ($protocol ne 'https');
     unless ($path =~ m{^/}) {
         $path = '/'.$path;
     }
-    my $url = $protocol.'://'.&Apache::lonnet::hostname($desthost).$path;
+    my $url = $protocol.'://'.$hostname.$path;
     if ($env{'form.firsturl'} ne '') {
         $url .='?firsturl='.$env{'form.firsturl'};
     }
