@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Cookie Based Access Handler
 #
-# $Id: publiccheck.pm,v 1.24.2.1 2017/06/06 22:37:18 raeburn Exp $
+# $Id: publiccheck.pm,v 1.24.2.2 2021/01/04 03:53:20 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -107,7 +107,7 @@ sub process_portfolio {
     my %access_controls = &Apache::lonnet::get_access_controls($current_perms,$group,$file_name);
     my $access = '';
     my $now = time;
-    my $clientip = $r->get_remote_host();
+    my $clientip = &Apache::lonnet::get_requestor_ip($r);
     foreach my $key (keys(%{$access_controls{$file_name}})) {
         my ($num,$scope,$end,$start) = ($key =~ /^([^:]+):([a-z]+)_(\d*)_?(\d*)$/);
         if ($start > $now) {
