@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Switch Servers Handler
 #
-# $Id: switchserver.pm,v 1.35 2016/12/16 15:31:11 raeburn Exp $
+# $Id: switchserver.pm,v 1.35.2.5 2021/01/04 03:54:38 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -31,7 +31,7 @@ package Apache::switchserver;
 use strict;
 use Apache::Constants qw(:common);
 use Apache::lonnet;
-use Apache::lonmenu;
+use Digest::MD5 qw(md5_hex);
 use CGI::Cookie();
 use Apache::lonlocal;
 use LONCAPA qw(:DEFAULT :match);
@@ -49,16 +49,20 @@ sub init_env {
     }
     my $lonidsdir=$r->dir_config('lonIDsDir');
     &Apache::lonnet::transfer_profile_to_env($lonidsdir,$handle);
-    
+
     return $r->dir_config('lonIDsDir')."/$handle.id";
 }
 
 sub do_redirect {
     my ($r,$url,$only_body,$extra_text) = @_;
     $r->send_http_header;
+    my $delay = 0.5;
+    if ($only_body) {
+        $delay = 0;
+    }
     my $start_page = 
 	&Apache::loncommon::start_page('Switching Server ...',undef,
-				       {'redirect'       => [0.5,$url],
+				       {'redirect'       => [$delay,$url],
 					'only_body'      => $only_body,});
     my $end_page   = &Apache::loncommon::end_page();
     $r->print($start_page.$extra_text.$end_page);
@@ -68,6 +72,24 @@ sub do_redirect {
     return OK;
 }
 
+sub balancer_cookieid {
+    my ($r,$desthost,$uname,$udom) = @_;
+    my @hosts = &Apache::lonnet::current_machine_ids();
+    my $newcookieid;
+    unless (grep(/^\Q$desthost\E$/,@hosts)) {
+        my $balancedir=$r->dir_config('lonBalanceDir');
+        $newcookieid = &md5_hex(&md5_hex(time.{}.rand().$$));
+        my $cookie = $udom.'_'.$uname.'_'.$newcookieid;
+        my $balcookie = "balanceID=$cookie; path=/; HttpOnly;";
+        if (open(my $fh,'>',"$balancedir/$cookie.id")) {
+            print $fh $desthost;
+            close($fh);
+            $r->headers_out->add('Set-cookie' => $balcookie);
+        }
+    }
+    return $newcookieid;
+}
+
 sub flush_course_logs {
     &Apache::lonnet::flushcourselogs();
     return OK;
@@ -101,20 +123,26 @@ sub handler {
         if ($Apache::lonnet::protocol{$env{'form.otherserver'}} eq 'https') {
             $protocol = $Apache::lonnet::protocol{$env{'form.otherserver'}};
         }
-    } 
+    }
 
     if ($env{'user.name'} eq 'public'
 	&& $env{'user.domain'} eq 'public') {
 	my $url = $protocol.'://'.$switch_to.$r->uri;
+        unlink($handle);
+        #expire the cookie
+        my $c = new CGI::Cookie(-name    => 'lonPubID',
+                                -value   => '',
+                                -expires => '-10y',);
+        $r->header_out('Set-cookie' => $c);
 	return &do_redirect($r,$url,1)
     }
 
     my $skip_canhost_check = '';
+    my $now = time;
     if ($env{'form.role'}) {
         if (!exists($env{'user.role.'.$env{'form.role'}})) {
             delete($env{'form.role'});
         } else {
-            my $now = time;
             my ($start,$end) = split(/\./,$env{'user.role.'.$env{'form.role'}});
             if (($start && $start > $now) || ($end && $end < $now)) {
                 delete($env{'form.role'});
@@ -156,8 +184,17 @@ sub handler {
     }
 
     #remove session env, and log event
-    unlink($handle);
-    my %temp=('switchserver' => time.':'.$env{'form.otherserver'},
+    if (unlink($handle)) {
+        if ($env{'user.linkedenv'} ne '') {
+            my $lonidsdir=$r->dir_config('lonIDsDir');
+            if (($env{'user.linkedenv'} =~ /^[a-f0-9]+_linked$/) &&
+                (-l "$lonidsdir/$env{'user.linkedenv'}.id") &&
+                (readlink("$lonidsdir/$env{'user.linkedenv'}.id") eq $handle)) {
+                unlink("$lonidsdir/$env{'user.linkedenv'}.id");
+            }
+        }
+    }
+    my %temp=('switchserver' => $now.':'.$env{'form.otherserver'},
 	      $env{'form.role'});
     &Apache::lonnet::put('email_status',\%temp);
     my $logmsg = "Switch Server to $env{'form.otherserver'}";
@@ -166,33 +203,63 @@ sub handler {
     } else {
         $logmsg .= " (no role)";
     }
-    $logmsg .= ' '.$ENV{'REMOTE_ADDR'};
+    my $ip = &Apache::lonnet::get_requestor_ip();
+    $logmsg .= ' '.$ip;
     &Apache::lonnet::log($env{'user.domain'},$env{'user.name'},
 			 $env{'user.home'},$logmsg);
 
     &Apache::loncommon::content_type($r,'text/html');
 
-    #expire the cookie
-    my $c = new CGI::Cookie(-name    => 'lonID',
-			    -value   => '',
-			    -expires => '-10y',);
-    $r->header_out('Set-cookie' => $c);
-
+    #expire the cookies
+    my %cookies=CGI::Cookie->parse($r->header_in('Cookie'));
+    foreach my $name (keys(%cookies)) {
+        next unless ($name =~ /^lon(|S|Link|Pub)ID$/);
+        my $c = new CGI::Cookie(-name    => $name,
+                                -value   => '',
+                                -expires => '-10y',);
+        $r->headers_out->add('Set-cookie' => $c);
+    }
     if ($r->header_only) {
 	$r->send_http_header;
 	return OK;
     }
 # -------------------------------------------------------- Menu script and info
-	   
+
 # ---------------------------------------------------------------- Get handover
 
-    my ($is_balancer) = &Apache::lonnet::check_loadbalancing($env{'user.name'},$env{'user.domain'});
-    my %info=('ip'       => $ENV{'REMOTE_ADDR'},
+    my ($is_balancer,$setcookie,$newcookieid,$otherbalcookie);
+    my $only_body = 0;
+    ($is_balancer,undef,$setcookie) =
+        &Apache::lonnet::check_loadbalancing($env{'user.name'},$env{'user.domain'},'switchserver');
+    if ($is_balancer && $setcookie && $env{'form.otherserver'}) {
+
+        # Set a balancer cookie unless browser already sent LON-CAPA load balancer
+        # cookie which points at the target server
+        my ($found_server,$balancer_cookie) = &Apache::lonnet::check_for_balancer_cookie($r,1);
+
+        if (($found_server eq $env{'form.otherserver'}) &&
+            ($balancer_cookie =~ /^\Q$env{'user.domain'}\E_\Q$env{'user.name'}\E_/)) {
+            $only_body = 1;
+        } else {
+            $newcookieid = &balancer_cookieid($r,$env{'form.otherserver'},
+                                              $env{'user.name'},$env{'user.domain'});
+        }
+    }
+    if ((!$is_balancer) && ($env{'request.balancercookie'})) {
+        $otherbalcookie = $env{'request.balancercookie'};
+    }
+
+    my %info=('ip'       => $ip,
 	      'domain'   => $env{'user.domain'},
 	      'username' => $env{'user.name'},
 	      'role'     => $env{'form.role'},
 	      'server'   => $r->dir_config('lonHostID'),
 	      'balancer' => $is_balancer);
+    if ($newcookieid) {
+        $info{'balcookie'} = $newcookieid;
+    } elsif ($otherbalcookie) {
+        $info{'otherbalcookie'} = $otherbalcookie;
+    }
     if ($env{'form.origurl'}) {
         $info{'origurl'} = $env{'form.origurl'};
     }
@@ -211,7 +278,7 @@ sub handler {
 	'&amp;username='.$env{'user.name'}.
 	'&amp;token='.$token;
 # --------------------------------------------------------------- Screen Output
-    return &do_redirect($r, $url, 0);
+    return &do_redirect($r, $url, $only_body);
 }
 
 1;
