@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 # Automated Userinfo update script
-# $Id: Autoupdate.pl,v 1.22 2011/12/18 01:23:21 raeburn Exp $
+# $Id: Autoupdate.pl,v 1.23 2016/01/27 22:23:18 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -198,7 +198,7 @@ sub descend_tree {
         closedir(DIR);
         $depth ++;
         foreach my $item (@contents) {
-            if ($depth < 4) {
+            if (($depth < 4) && (length($item) == 1)) {
                 &descend_tree($dom,$dir.'/'.$item,$depth,$alldomusers,$coursesref);
             } elsif (-e $dir.'/'.$item.'/passwd') {
                 if (ref($coursesref) eq 'HASH') {
