@@ -1,6 +1,6 @@
 # functions to glue school database system into Lon-CAPA for 
 # automated enrollment
-# $Id: localenroll.pm,v 1.56 2016/08/25 18:01:34 raeburn Exp $
+# $Id: localenroll.pm,v 1.59 2018/06/14 04:11:49 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -39,8 +39,6 @@ described at http://www.lon-capa.org.
 
 =head1 NOTABLE SUBROUTINES
 
-=over
-
 =cut
 
 package localenroll;
@@ -48,6 +46,8 @@ package localenroll;
 use strict;
 
 =pod
+
+=over
  
 =item run()
  set this to return 1 if you want the auto enrollment to run
@@ -258,9 +258,11 @@ sub get_sections {
  username:domain
  (c) the LON-CAPA domain that contains the course
 
- new_course also takes a fourth (optional) argument -
+ new_course also takes optional fourth and fifth arguments -
  (d) the course co-owners, as a comma-separated list of username:domain for
- any co-owners. 
+ any co-owners.
+ (e) database handle (might be set when new_course() is called by check_section
+ routine within localenroll.pm).
 
 =cut
 
@@ -588,6 +590,46 @@ sub export_grades {
     }
 }
 
+=pod
+
+=item check_instclasses()
+
+ This is used to supply information about which instituional course sections
+ and cross-listings are available to supply enrollment data, given the current
+ list of owner and co-owners. The data are used to populate the column titled:
+ "Auto-enrollment of registered students" when showing full detailed for a course
+ in the course catalog.
+
+ This subroutine takes four arguments -
+
+ (a) $owners - comma-separated list of username:domain for course owner 
+     and co-owners.
+ (b) $dom - domain of course.
+ (c) $classes - reference to hash of institutional course sections and
+     crosslistings for which access to enrollment data is being checked.
+ (d) $validated - reference to hash which will be populated with all
+     keys from incoming $classes hashref, for which one or more of the
+     owner/co-owners has rights to access enrollment data. For each
+     key included in $validated hashref, corresponding value will be set to 1.
+  
+ The subroutine returns 'ok' if there is no processing error.
+
+=cut
+
+
+sub check_instclasses {
+    my ($owners,$dom,$classes,$validated) = @_;
+    if ((ref($classes) eq 'HASH') && (ref($validated) eq 'HASH')) {
+        foreach my $class (keys(%{$classes})){
+            if (&check_section($class,$owners,$dom) eq 'ok') {
+                $validated->{$class} = 1;
+            }
+        }
+    }
+    return 'ok';
+}
+
+
 =pod
 
 =item create_password()
@@ -955,8 +997,8 @@ sub allusers_info {
               institutional types to check.
  (g) $srchby - optional if $uname or $id defined, otherwise required.
                Allowed values include: 1. lastfirst, 2. last, 3. uname
-               corresponding to searches by 1. lastname,firstname;
-               2. lastname; 3. username
+               4. email, corresponding to searches by 1. lastname,firstname;
+               2. lastname; 3. username; 4. e-mail address
  (h) $srchterm - optional if $uname or $id defined, otherwise required
                 String to search for.
  (i) $srchtype - optional. Allowed values: contains, begins (defaults
