@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # caparesponse definition
 #
-# $Id: caparesponse.pm,v 1.258 2016/08/09 23:43:45 raeburn Exp $
+# $Id: caparesponse.pm,v 1.261 2020/12/04 22:19:19 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -664,7 +664,14 @@ sub end_numericalresponse {
 	my (@formats)=&Apache::lonxml::get_param_var('format',$parstack,$safeeval);
 
 	my $unit=&Apache::lonxml::get_param_var('unit',$parstack,$safeeval);
-	
+	my ($needsdollar,$needscomma);
+	if ($unit =~ /\$/) {
+	    $needsdollar = 1;
+	}
+	if ($unit =~ /\,/) {
+	    $needscomma = 1;
+	}
+
 	if ($target eq 'answer') {
 	    $result.=&Apache::response::answer_header($tag,undef,
 						      scalar(keys(%answer)));
@@ -753,17 +760,21 @@ sub end_numericalresponse {
 		my $ans=$answers[$i];
 		my $fmt=$formats[0];
 		if (@formats && $#formats) {$fmt=$formats[$i];}
-		foreach my $element (@$ans) {		    
+		my @answers;
+		if (ref($ans) eq 'ARRAY') {
+		    @answers = (@{$ans});
+		}
+		foreach my $element (@answers) {		    
 		    if ($fmt && $tag eq 'numericalresponse') {
 			$fmt=~s/e/E/g;
 			if ($unit=~/\$/) { $fmt="\$".$fmt; $unit=~s/\$//g; }
 			if ($unit=~/\,/) { $fmt="\,".$fmt; $unit=~s/\,//g; }
 			$element = &format_number($element,$fmt,$target,
 						  $safeeval);
-			if ($fmt=~/\$/ && $unit!~/\$/) { $element=~s/\$//; }
+			if ($fmt=~/\$/ && !$needsdollar) { $element=~s/\$//; }
 		    }
 		}
-		push(@fmt_ans,join(',',@$ans));
+		push(@fmt_ans,join(',',@answers));
 	    }
 	    my $response=\@fmt_ans;
 
@@ -773,7 +784,7 @@ sub end_numericalresponse {
 		! ($Apache::lonhomework::type eq 'exam' ||
 		   lc($hideunit) eq "yes") )  {
 		my $cleanunit=$unit;
-		$cleanunit=~s/\$\,//g;
+		$cleanunit=~s/[\$,]//g;
 		foreach my $ans (@fmt_ans) {
 		    $ans.=" $cleanunit";
 		}
@@ -790,7 +801,13 @@ sub end_numericalresponse {
 						    $tag,$parstack,
 						    $safeeval,1);
 		    $error=&mt("Computer's answer is incorrect ([_1]).",'"'.join(', ',@$response).'"').' ';
-                    if ($ad eq 'UNIT_INVALID_STUDENT') {
+                    if (($ad eq 'NO_UNIT') && $needsdollar) {
+                        $error.=&mt('The unit attribute includes [_1] but the answer format does not.','$').' '.
+                                &mt('Either remove the [_1] from the unit or prepend [_1] to the answer format.','$');    
+                    } elsif (($ad eq 'COMMA_FAIL') && $needscomma) {
+                        $error.=&mt('The unit attribute includes [_1] but the answer format does not.',',').' '.
+                                &mt('Either remove the [_1] from the unit or prepend [_1] to the answer format.',',');
+                    } elsif ($ad eq 'UNIT_INVALID_STUDENT') {
                         $error.=&mt('Unable to interpret units. Computer reads units as "[_1]".',$msg).' '.
                                 &mt('The unit attribute in the numericalresponse item needs to be a supported physical unit.');
 		    } elsif ($sigline ne '') {
@@ -1048,7 +1065,12 @@ sub make_numerical_bubbles {
                      &Math::Random::random_uniform_integer(1,1,10);
     }
     for ($ind=0;$ind<$number_of_bubbles;$ind++) {
-	$bubble_values[$ind] = $answerfactor*($factor**($power-$powers[$#powers-$ind]));
+	my $exponent = $power-$powers[$#powers-$ind];
+	if ($exponent == 0) {
+	    $bubble_values[$ind] = $answerfactor;
+	} else { 
+	    $bubble_values[$ind] = $answerfactor*($factor**$exponent);
+	}
 	$bubble_display[$ind] = &format_number($bubble_values[$ind],
 					       $format,$target,$safeeval);
     }
