@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # input  definitons
 #
-# $Id: inputtags.pm,v 1.333.2.4 2016/08/05 23:17:10 raeburn Exp $
+# $Id: inputtags.pm,v 1.333.2.9 2019/12/04 02:10:23 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -666,10 +666,13 @@ sub file_selector {
     if ($constraints) {
         $result .= $constraints.'<br />';
     }
-    if ($which eq 'uploadonly' || $which eq 'both') { 
-	$result.=&mt('Submit a file: (only one file per submission)').
-	    ' <br /><input type="file" size="50" name="HWFILE'.
-	    $jspart.'_'.$id.'" id="HWFILE'.$jspart.'_'.$id.'" /><br />';
+    if ($which eq 'uploadonly' || $which eq 'both') {
+        my $free_space = $maxfilesize * 1048576;
+        $result .= &mt('Submit a file: (only one file per submission)').
+            ' <br /><input type="file" size="50" name="HWFILE'.$jspart.'_'.$id.
+            '" id="HWFILE'.$jspart.'_'.$id.'" class="LC_flUpload LC_hwkfile" />'.
+            '<input type="hidden" id="LC_free_space_'.$jspart.'_'.$id.'"'.
+            ' value="'.$free_space.'" /><br />';
     }
     if ( $which eq 'both') {
 	$result.='<br />'.'<strong>'.&mt('OR:').'</strong><br />';
@@ -691,7 +694,6 @@ sub file_selector {
 	    &mt('Select Portfolio Files: (one or more files per submission)').'</a><br />'.
 	    '<input type="text" size="50" name="HWPORT'.$jspart.'_'.$id.'" value="" />'.
 	    '<br />';
-
     }
     $result.=&Apache::lonhtmlcommon::row_closure(1);
     return $result;
@@ -1035,6 +1037,7 @@ sub decideoutput {
 	  'not_charged_try' => 'LC_answer_not_charged_try',
 	  'no_grade'        => 'LC_answer_no_grade',
 	  'no_message'      => 'LC_no_message',
+          'no_charge_warn'  => 'LC_answer_warning',
 	  );
 
     my $part = $Apache::inputtags::part;
@@ -1198,8 +1201,12 @@ sub decideoutput {
     } elsif ($award eq 'SIG_FAIL') {
 	my ($used,$min,$max)=split(':',$awardmsg);
 	my $word = ($used < $min) ? 'more' : 'fewer';
-	$message = &mt("Submission not graded. Use $word digits.",$used);
-	$css_class=$possible_class{'not_charged_try'};
+	$message = &mt("Submission not graded. Use $word significant figures.",$used);
+        if (&Apache::lonhomework::show_some_problem_status()) {
+            $css_class=$possible_class{'no_charge_warn'};
+        } else {
+            $css_class=$possible_class{'not_charged_try'};
+        }
 	$button=1;
     } elsif ($award eq 'UNIT_INVALID_INSTRUCTOR') {
 	$message = &mt('Error in instructor specifed unit. This error has been reported to the instructor.', $awardmsg);
@@ -1871,8 +1878,13 @@ sub previous_tries {
     my $tries_text = &get_tries_text('link');
     my $prefix = $env{'form.request.prefix'};
     $prefix =~ tr{.}{_};
-    my $function_name = "LONCAPA_previous_tries_".$prefix.
-	$Apache::lonxml::curdepth.'_'.$env{'form.counter'};
+    my $function_name = 'LONCAPA_previous_tries_'.$prefix;
+    if (($env{'request.state'} eq 'construct') || ($id =~ /[._]|[^\w\s\-]/)) {
+        $function_name .= $Apache::lonxml::curdepth;
+    } else {
+        $function_name .= &js_escape($id);
+    }
+    $function_name .= '_'.$Apache::lonxml::counter;
     my $result = &Apache::loncommon::modal_adhoc_window($function_name,420,410,$output,&mt($tries_text))."<br />";
     return $result;
 }
