@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Interface routines to MAXIMA CAS
 #
-# $Id: lonmaxima.pm,v 1.30 2012/02/17 15:31:04 raeburn Exp $
+# $Id: lonmaxima.pm,v 1.38 2017/06/12 00:22:35 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -118,7 +118,7 @@ sub runscript {
 sub maxima_cas_formula_fix {
    my ($expression)=@_;
    $expression=&Apache::response::implicit_multiplication($expression);
-   $expression=~s/([^\w\%])pi/$1\%pi/gs;
+   $expression=~s/(^|[^\w\%])pi(\W|$)/$1\%pi$2/gs;
    return $expression;
 }
 
@@ -155,7 +155,7 @@ sub maxima_eval {
 
 sub compareterms {
     my ($socket,$terma,$termb)=@_;
-    my $difference=$terma.'-('.$termb.')';
+    my $difference='('.$terma.')-('.$termb.')';
     if (&blacklisted($difference)) { return 'Error: blacklisted'; }
     my $reply=&maximareply($socket,'trigsimp(trigreduce('.$difference.'));');
     if ($reply=~/^\s*0\s*$/) { return 'true'; }
@@ -166,7 +166,10 @@ sub compareterms {
 sub maxima_check {
     my ($response,$answer,$reterror) = @_;
     my $socket=&connect();
-    my $reply=&compareterms($socket,$response,$answer);
+    my $reply=&maximareply($socket,$response.';');
+    unless ($reply=~/^Error\:/) {
+        $reply=&compareterms($socket,$response,$answer);
+    }
     &disconnect($socket);
     # integer to string mappings come from capaParser.h
     # 1 maps to 'EXACT_ANS'
