@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # rank style response
 #
-# $Id: rankresponse.pm,v 1.72 2015/01/19 15:35:53 goltermann Exp $
+# $Id: rankresponse.pm,v 1.72.2.1 2018/06/08 11:57:01 raeburn Exp $
 # Copyright Michigan State University Board of Trustees
 #
 # This file is part of the LearningOnline Network with CAPA (LON-CAPA).
@@ -410,7 +410,7 @@ sub displayfoils {
 	    my $lastopt=$lastresponse{$name};
 	    my $optionlist='';
 	    if ($target ne 'tex') {
-                $optionlist = "<option></option>\n";
+                $optionlist = "<option value=\"\"></option>\n";
             } 
             if ($target eq 'tex' && $env{'form.pdfFormFields'} eq 'yes') {
                 my $fieldname = $env{'request.symb'}.'&part_'.$Apache::inputtags::part.'&rankresponse'.'&HWVAL_'.$Apache::inputtags::response['-1'].':'.$temp;
@@ -420,13 +420,13 @@ sub displayfoils {
 	    foreach $option (@whichopt) {
 		if ($option eq $lastopt) {
 		    if ($target ne 'tex' ) {
-                        $optionlist.="<option selected=\"selected\">$option</option>\n";
+                        $optionlist.="<option value=\"$option\" selected=\"selected\">$option</option>\n";
                     } elsif ($target eq 'tex' && $env{'form.pdfFormFields'} eq 'yes') {
                         $optionlist .= &Apache::lonxml::print_pdf_add_combobox_option($option); 
                     }
 		} else {
 		    if ($target ne 'tex') {
-                        $optionlist.="<option>$option</option>\n";
+                        $optionlist.="<option value=\"$option\">$option</option>\n";
                     } elsif ($target eq 'tex' && $env{'form.pdfFormFields'} eq 'yes') {
                         $optionlist .= &Apache::lonxml::print_pdf_add_combobox_option($option); 
                     }
