@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # This will take annotations and then plug them into a page.
 #
-# $Id: admannotations.pm,v 1.28.2.2 2009/01/08 19:55:25 raeburn Exp $
+# $Id: admannotations.pm,v 1.36 2018/12/29 22:31:16 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -27,6 +27,7 @@
 #
 ##################
 
+
 package Apache::admannotations;
 
 use strict;
@@ -74,6 +75,12 @@ sub construct_editor {
 
     my $get_symb = &js_get_symb();
 
+    my %lt = &Apache::lonlocal::texthash(
+        'close no save' => 'Close',
+        'save'          => 'Save',
+        'cancel'        => 'Cancel',
+    );
+
     my $js = <<ENDJS;
 <script type="text/javascript">
 var timeout;
@@ -88,6 +95,11 @@ function changed() {
     }
     timeout=setTimeout('changed();','1000');
 }
+
+function renameButton(){
+    document.getElementById("close").value="$lt{'cancel'}";
+}
+
 </script>
 ENDJS
 
@@ -101,20 +113,17 @@ ENDJS
     my $end_page =
         &Apache::loncommon::end_page();
     
-    my %lt = ( 'close no save' => "Close (no save)",
-	       'save'          => "Save and Update",);
-    %lt = &Apache::lonlocal::texthash(%lt);
 	 
     return(<<END_HTML)
 $start_page
  <center>
-  <form name="annotInfo" method="post" action="/adm/annotations">
-    <textarea name="annotation" wrap="on" rows="12" cols="36">$annotation</textarea><br />
+  <form name="annotInfo" method="post" action="">
+    <textarea name="annotation" wrap="on" rows="12" cols="36" onfocus="javascript:renameButton();">$annotation</textarea><br />
     <input type="hidden" name="symbold" value="$symbnew" />
     <input type="hidden" name="symbnew" value="" />
     <input type="button" name="send" value="$lt{'save'}"
            onclick="javascript:this.form.symbnew.value=get_symb();this.form.submit();" />
-    <input type="button" name="close" value="$lt{'close no save'}" 
+    <input type="button" id="close" name="close" value="$lt{'close no save'}" 
            onclick="javascript:window.close();" />
   </form>
  </center>
@@ -138,14 +147,15 @@ ENDJS
 						    {'only_body'   => 1,});
     my $end_page =  &Apache::loncommon::end_page();
 
-    my %lt = ( 'close'  => "Close",
-	       'button' => $button_name,);
-    %lt = &Apache::lonlocal::texthash(%lt);
+    my %lt = &Apache::lonlocal::texthash(
+        'close'  => 'Close',
+        'button' => $button_name,
+    );
 
     return(<<END_HTML2)
 $start_page
  <center>
-  <form name="annotInfo" method="post" action="/adm/annotations">
+  <form name="annotInfo" method="post" action="">
 <table bgcolor="#FFFFFF" width="100%" height="90%" align="center">
 <td>
 <font size="+1"><i>
@@ -175,7 +185,6 @@ sub handler {
     $r->send_http_header;
     return OK if $r->header_only;
 
-
     my $page;
 
     my ($symb_old,$symb_old_enc) = &Apache::loncommon::clean_symb($env{'form.symbold'},1);
@@ -204,3 +213,41 @@ sub handler {
 1;
 __END__
 
+=pod
+
+=head1 NAME
+
+Apache::admannotations
+
+=head1 SYNOPSIS
+
+This will take annotations and then plug them into a page.
+
+This is part of the LearningOnline Network with CAPA project
+described at http://www.lon-capa.org.
+
+=head1 OVERVIEW
+
+(empty)
+
+=head1 SUBROUTINES
+
+=over write_annotation()
+
+Put annotation
+
+=item js_get_symb()
+
+Construct editor
+
+=item construct_editor()
+
+=item construct_error()
+
+=item handler()
+
+=back
+
+=cut
+
+
