@@ -1,6 +1,6 @@
 # The LearningOnline Network with CAPA
 # 
-# $Id: imsimport.pm,v 1.47 2015/06/09 21:22:51 damieng Exp $
+# $Id: imsimport.pm,v 1.48 2017/11/05 20:04:30 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -38,6 +38,7 @@ use HTML::Entities();
 use Apache::lonlocal;
 use Apache::lonupload;
 use File::Basename();
+use File::Path();
 use LONCAPA;
 
 # ----------------------------------------------------------------  Jscript One
@@ -640,7 +641,7 @@ sub display_three {
         );
         my $londocroot = $r->dir_config('lonDocRoot');
         if ($destdir =~ m{^\Q$londocroot/priv/$udom/$uname/$dirpath\E}) {
-            system (" rm -r -f $destdir/temp");
+            &File::Path::remove_tree("$destdir/temp",{ safe => 1 });
         }
     } elsif ($manifest_result eq 'nomanifest') {
         $r->print(
