@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Handler for displaying the course catalog interface
 #
-# $Id: coursecatalog.pm,v 1.88.2.1 2016/10/23 21:21:56 raeburn Exp $
+# $Id: coursecatalog.pm,v 1.88.2.7 2019/08/02 00:24:02 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -77,24 +77,23 @@ sub handler {
     my $domdesc = &Apache::lonnet::domain($codedom,'description');
     &Apache::lonhtmlcommon::clear_breadcrumbs();
 
-    my %domconfig =
-        &Apache::lonnet::get_dom('configuration',['coursecategories'],$codedom);
+    my $crscats = &Apache::lonnet::get_dom_cats($codedom);
+    my %domdefaults = &Apache::lonnet::get_domain_defaults($codedom);
     my $knownuser = &user_is_known();
     my $canviewall = &canview_all($knownuser,$codedom);
     
     my ($cathash,$cattype);
-    if (ref($domconfig{'coursecategories'}) eq 'HASH') {
-        $cathash = $domconfig{'coursecategories'}{'cats'};
-        if ($knownuser || $canviewall) {
-            $cattype = $domconfig{'coursecategories'}{'auth'};
-        } else {
-            $cattype = $domconfig{'coursecategories'}{'unauth'};
-        }
-        if ($cattype eq '') {
-            $cattype = 'std';
-        }
+    if (ref($crscats) eq 'HASH') {
+        %{$cathash} = %{$crscats};
     } else {
         $cathash = {};
+    }
+    if ($knownuser || $canviewall) {
+        $cattype = $domdefaults{'catauth'};
+    } else {
+        $cattype = $domdefaults{'catunauth'};
+    }
+    if ($cattype eq '') {
         $cattype = 'std';
     }
     if ($cattype eq 'none') {
@@ -106,7 +105,7 @@ sub handler {
         if ($knownuser || $canviewall) {
             $r->print('<div>'.&mt('No catalog of LON-CAPA courses/communities is provided for: [_1]',$domdesc).'</div>');
         } else {
-            if ($domconfig{'coursecategories'}{'auth'} eq 'none') {
+            if ($domdefaults{'catauth'} eq 'none') {
                 $r->print('<div>'.&mt('No catalog of LON-CAPA courses/communities is provided for: [_1]',$domdesc).'</div>');
             } else {
                 $r->print('<div>'.&mt('The catalog of LON-CAPA courses/communities provided for: "[_1]" is only available to users who are logged in.',$domdesc).'</div>');
@@ -116,7 +115,7 @@ sub handler {
         return OK;
     }
 
-    my $cnum; 
+    my $cnum;
     if ($cattype eq 'codesrch') {
         my ($uniquecode,$codemsg,$brtext);
         if ($env{'form.uniquecode'}) {
@@ -125,7 +124,7 @@ sub handler {
         }
         my $js = '<script type="text/javascript">'."\n".
                  '// <![CDATA['."\n".
-                 &courselink_javascript()."\n".
+                 &courselink_javascript($r)."\n".
                  '// ]]>'."\n".
                  '</script>'."\n";
         $r->print(&Apache::loncommon::start_page('Search for a Course/Community',$js));
@@ -176,8 +175,14 @@ sub handler {
                                       $courseinfo{$codedom.'_'.$cnum}{$item});
                             if ($item eq 'title') {
                                 if ($courseinfo{$codedom.'_'.$cnum}{'showsyllabus'}) {
+                                    my $usehttp = 0;
+                                    if (($ENV{'SERVER_PORT'} == 443) && ($courseinfo{$codedom.'_'.$cnum}{'extsyllplain'})) {
+                                        unless (&Apache::lonnet::uses_sts()) {
+                                            $usehttp = 1;
+                                        }
+                                    }
                                     $r->print('&nbsp;<font size="-2">'.
-                                              '<a href="javascript:ToSyllabus('."'$codedom','$cnum'".')">'.
+                                              '<a href="javascript:ToSyllabus('."'$codedom','$cnum','$usehttp'".')">'.
                                               &mt('Syllabus').'</a></font>');
                                 }
                             }
@@ -207,21 +212,31 @@ sub handler {
         $r->print('<br />'.&Apache::loncommon::end_page());
         return OK;
     } else {
-        if ($env{'form.coursenum'}) {
-            $cnum = $env{'form.coursenum'};
+        if ($env{'form.coursenum'} ne '') {
+            if ($env{'form.coursenum'} =~ /^$LONCAPA::match_courseid$/) {
+                $cnum = $env{'form.coursenum'};
+            } else {
+                delete($env{'form.coursenum'});
+            }
         }
     }
 
-    if ($env{'form.catalog_maxdepth'} ne '') {
-        $env{'form.catalog_maxdepth'} =~ s{\D}{}g;
-    }
-
-    my (@cats,@trails,%allitems,%idx,@jsarray,%subcathash,$subcats);
+    my (@cats,@trails,%allitems,%idx,@jsarray,%subcathash,$subcats,%maxd,
+        $toplevelstr,$maxdepthstr);
     if ($env{'form.withsubcats'}) {
         $subcats = \%subcathash;
     }
     &Apache::loncommon::extract_categories($cathash,\@cats,\@trails,\%allitems,
-                                           \%idx,\@jsarray,$subcats);
+                                           \%idx,\@jsarray,$subcats,\%maxd);
+    if (ref($cats[0]) eq 'ARRAY') {
+        foreach my $item (@{$cats[0]}) {
+            $toplevelstr .= "'".&js_escape($item)."::0',";
+            $maxdepthstr .= "'$maxd{$item}',";
+        }
+        $toplevelstr =~ s/,$//;
+        $maxdepthstr =~ s/,$//;
+    }
+    &validate_input($codedom,\@cats,\%maxd,$cathash);
     my ($numtitles,@codetitles);
     if (($env{'form.coursenum'} ne '') && ($knownuser)) {
         &course_details($r,$codedom,$formname,$domdesc,\@trails,\%allitems,\@codetitles);
@@ -231,9 +246,29 @@ sub handler {
         my $catjs = <<"ENDSCRIPT";
 
 function setCatDepth(depth) {
-    document.coursecats.catalog_maxdepth.value = depth;
-    if (depth == '') {
+    var depth = parseInt(depth);
+    if (depth !== NaN) {
+        if (depth > 0) {
+            var possmaxd = 0;
+            var toplevel = new Array($toplevelstr);
+            var maxdepths = new Array($maxdepthstr);
+            if (toplevel.length) {
+                for (var i=0; i<toplevel.length; i++) {
+                    var item = unescape(toplevel[i]);
+                    if (item == document.coursecats.currcat_0.value) {
+                        possmaxd = maxdepths[i];
+                        break;
+                    }
+                }
+            }
+            if (depth > possmaxd) {
+                depth = possmaxd;
+            }
+        }
+        document.coursecats.catalog_maxdepth.value = depth;
+    } else {
         document.coursecats.currcat_0.value = '';
+        document.coursecats.catalog_maxdepth.value = '';
     }
     document.coursecats.submit();
     return;
@@ -250,7 +285,7 @@ function setCourseId(caller) {
 }
 
 ENDSCRIPT
-        $catjs .= &courselink_javascript();
+        $catjs .= &courselink_javascript($r);
         if (&user_is_dc($codedom) || $canviewall) {
             $catjs .= <<ENDTOGGJS
 
@@ -340,13 +375,280 @@ ENDJS
     return OK;
 }
 
+sub validate_input {
+    my ($codedom,$cats,$maxd,$crscatsref) = @_;
+    my $currcat = '';
+    my $depth = 0;
+    if ($env{'form.catalog_maxdepth'} ne '') {
+        $env{'form.catalog_maxdepth'} =~ s{\D}{}g;
+    }
+    if ((ref($cats) eq 'ARRAY') && (ref($maxd) eq 'HASH')) {
+        if (ref($cats->[0]) eq 'ARRAY') {
+            if (@{$cats->[0]} == 1) {
+                if ($cats->[0][0] eq 'instcode') {
+                    $currcat = 'instcode::0';
+                } elsif ($cats->[0][0] eq 'communities') {
+                    $currcat = 'communities::0';
+                } elsif ($cats->[0][0] eq 'placement') {
+                    $currcat = 'placement::0';
+                } else {
+                    my $name = $cats->[0][0];
+                    $currcat = &escape($name).'::0';
+                }
+                if (exists($maxd->{$cats->[0][0]})) {
+                    if ($env{'form.catalog_maxdepth'} <= $maxd->{$cats->[0][0]}) {
+                        $depth = $env{'form.catalog_maxdepth'};
+                    } else {
+                        $depth = $maxd->{$cats->[0][0]};
+                    }
+                }
+            } elsif ((@{$cats->[0]} > 1) && ($env{'form.currcat_0'} ne '')) {
+                my ($escname) = ($env{'form.currcat_0'} =~ /^([^:]+)\:\:0$/);
+                if ($escname =~ /^instcode|communities|placement$/) {
+                    $currcat = $env{'form.currcat_0'};
+                    if (exists($maxd->{$escname})) {
+                        if ($env{'form.catalog_maxdepth'} <= $maxd->{$escname}) {
+                            $depth = $env{'form.catalog_maxdepth'};
+                        } else {
+                            $depth = $maxd->{$escname};
+                        }
+                    } else {
+                        $depth = 1;
+                    }
+                } elsif ($escname ne '') {
+                    my $name = &unescape($escname);
+                    if (grep(/^\Q$name\E$/,@{$cats->[0]})) {
+                        $currcat = $env{'form.currcat_0'};
+                        if (exists($maxd->{$name})) {
+                            if ($env{'form.catalog_maxdepth'} <= $maxd->{$name}) {
+                                $depth = $env{'form.catalog_maxdepth'};
+                            } else {
+                                $depth = $maxd->{$name};
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+    $env{'form.currcat_0'} = $currcat;
+    $env{'form.catalog_maxdepth'} = $depth;
+
+    my %possibles = (
+        backto => 1,
+        catalogfilter => 1,
+        catalog_maxdepth => 1,
+        courseid => 1,
+        coursenum => 1,
+        currcat_0 => 1,
+        numtitles => 1,
+        Year => 1,
+        Semester => 1,
+        Department => 1,
+        Number => 1,
+        showdom => 1,
+        sortby => 1,
+        showcounts => 1,
+        showdetails => 1,
+        showhidden => 1,
+        showselfenroll => 1,
+        state => 1,
+        uniquecode => 1,
+        wasactive => 1,
+        wasactiveon_day => 1,
+        wasactiveon_month => 1,
+        wasactiveon_year => 1,
+        withsubcats => 1,
+    );
+    my %currcats;
+    if ($env{'form.catalog_maxdepth'} > 0) {
+        for (my $i=1; $i<=$env{'form.catalog_maxdepth'}; $i++) {
+            $currcats{'currcat_'.$i} = 1;
+            $possibles{'currcat_'.$i} = 1;
+        }
+    }
+
+    foreach my $key (sort(keys(%env))) {
+        if ($key =~ /^form\.(.+)$/) {
+            unless ($possibles{$1}) {
+                delete($env{$key});
+            }
+        }
+    }
+    if (exists($env{'form.backto'})) {
+        unless ($env{'form.backto'} eq 'coursecatalog') {
+            delete($env{'form.backto'});
+        }
+    }
+    if (exists($env{'form.catalogfilter'})) {
+        unless (($env{'form.catalogfilter'} eq &mt('Display courses')) ||
+                ($env{'form.catalogfilter'} eq &mt('Display communities')) ||
+                ($env{'form.catalogfilter'} eq &mt('Display placement tests'))) {
+            delete($env{'form.catalogfilter'});
+        }
+    }
+    if (exists($env{'form.courseid'})) {
+        if ($env{'form.courseid'} ne '') {
+            unless ($env{'form.courseid'} =~ /^\Q$codedom\E_$LONCAPA::match_courseid$/) {
+                $env{'form.courseid'} = '';
+            }
+        }
+    }
+    if (exists($env{'form.coursenum'})) {
+        unless ($env{'form.coursenum'} =~ /^$LONCAPA::match_courseid$/) {
+            $env{'form.coursenum'} = '';
+        }
+    }
+    if (exists($env{'form.currcat_0'})) {
+        unless ($env{'form.currcat_0'} =~ /^(instcode|communities|placement)\Q::0\E$/) {
+            if ($env{'form.currcat_0'} =~ /^.+\Q::0\E$/) {
+                if (ref($crscatsref) eq 'HASH') {
+                    unless (exists($crscatsref->{$env{'form.currcat_0'}})) {
+                        delete($env{'form.currcat_0'});
+                    }
+                } else {
+                    delete($env{'form.currcat_0'});
+                }
+            } else {
+                delete($env{'form.currcat_0'});
+            }
+        }
+    }
+    if (exists($env{'form.numtitles'})) {
+        unless ($env{'form.numtitles'} =~ /^\d+$/) {
+            delete($env{'form.numtitles'});
+        }
+    }
+    my ($gotcodes,%possvals);
+    foreach my $item ('Year','Semester','Department','Number') {
+        if (exists($env{'form.'.$item})) {
+            unless ($env{'form.'.$item} eq '0') {
+                unless ($gotcodes) {
+                    $gotcodes = &get_instcode_allowable($codedom,\%possvals);
+                }
+                if (ref($possvals{$item}) eq 'HASH') {
+                    unless (exists($possvals{$item}{$env{'form.'.$item}})) {
+                        delete($env{'form.'.$item});
+                    }
+                } else {
+                    delete($env{'form.'.$item});
+                }
+            }
+        }
+    }
+    if (exists($env{'form.showdom'})) {
+        unless ($env{'form.showdom'} =~ /^$LONCAPA::match_domain$/) {
+            delete($env{'form.showdom'});
+        }
+    }
+    if (exists($env{'form.sortby'})) {
+        unless ($env{'form.sortby'} =~ /^(title|code|owner|cats)$/) {
+            delete($env{'form.sortby'});
+        }
+    }
+    if (exists($env{'form.showcounts'})) {
+        if (ref($env{'form.showcounts'}) eq 'ARRAY') {
+            foreach my $item (@{$env{'form.showcounts'}}) {
+                unless ($item =~ /^(Active|Future|Previous)$/) {
+                    delete($env{'form.showcounts'});
+                    last;
+                }
+            }
+        } else {
+            unless ($env{'form.showcounts'} =~ /^(Active|Future|Previous)$/) {
+                delete($env{'form.showcounts'});
+            }
+        }
+    }
+    foreach my $item ('showhidden','showdetails','showselfenroll','withsubcats') {
+        if (exists($env{'form.'.$item})) {
+            unless ($env{'form.'.$item} eq '1') {
+                delete($env{'form.'.$item});
+            }
+        }
+    }
+    if (exists($env{'form.state'})) {
+        unless ($env{'form.state'} eq 'listing') {
+            delete($env{'form.state'});
+        }
+    }
+    if (exists($env{'form.uniquecode'})) {
+        unless ($env{'form.uniquecode'} =~ /^\w{6}$/) {
+            delete($env{'form.uniquecode'});
+        }
+    }
+    if (exists($env{'form.wasactive'})) {
+        unless (($env{'form.wasactive'} eq 'accessend') || ($env{'form.wasactive'} eq 'enrollend')) {
+            delete($env{'form.wasactive'});
+        }
+    }
+    if (exists($env{'form.wasactiveon_day'})) {
+        my $tmpday = $env{'form.wasactiveon_day'};
+        unless (($tmpday =~ /^\d+$/) && ($tmpday > 0) && ($tmpday < 32)) {
+            delete($env{'form.wasactiveon_day'});
+        }
+    }
+    if (exists($env{'form.wasactiveon_month'})) {
+        my $tmpmonth = $env{'form.wasactiveon_month'};
+        unless (($tmpmonth =~ /^\d+$/) && ($tmpmonth > 0) && ($tmpmonth < 13)) {
+            delete($env{'form.wasactiveon_month'});
+        }
+    }
+    if (exists($env{'form.wasactiveon_year'})) {
+        my $tmpyear = $env{'form.wasactiveon_year'};
+        unless (($tmpyear =~ /^\d+$/) && ($tmpyear >= 1970)) {
+            delete($env{'form.wasactiveon_year'});
+        }
+    }
+    if (keys(%currcats)) {
+        foreach my $key (keys(%currcats)) {
+            if ($env{'form.'.$key} eq '') {
+                delete($env{'form.'.$key});
+            } elsif (ref($crscatsref) eq 'HASH') {
+                unless (exists($crscatsref->{$env{'form.'.$key}})) {
+                    delete($env{'form.'.$key});
+                }
+            } else {
+                delete($env{'form.'.$key});
+            }
+        }
+    }
+    return;
+}
+
+sub get_instcode_allowable {
+    my ($codedom,$possvalref) = @_;
+    return 1 unless (ref($possvalref) eq 'HASH');
+    my $instcats = &Apache::lonnet::get_dom_instcats($codedom);
+    if (ref($instcats) eq 'HASH') {
+        if ((ref($instcats->{'codetitles'}) eq 'ARRAY') && (ref($instcats->{'codes'}) eq 'HASH')) {
+            my %codes = %{$instcats->{'codes'}};
+            my @codetitles = @{$instcats->{'codetitles'}};
+            my @standardnames = &Apache::loncommon::get_standard_codeitems();
+            my %local_to_standard;
+            for (my $i=0; $i<@codetitles; $i++) {
+                $local_to_standard{$codetitles[$i]} = $standardnames[$i];
+                $possvalref->{$standardnames[$i]} = {};
+            }
+            foreach my $cid (sort(keys(%codes))) {
+                if (ref($codes{$cid}) eq 'HASH') {
+                    foreach my $item (keys(%{$codes{$cid}})) {
+                        $possvalref->{$local_to_standard{$item}}{$codes{$cid}{$item}} = 1;
+                    }
+                }
+            }
+        }
+    }
+    return 1;
+}
+
 sub course_details {
     my ($r,$codedom,$formname,$domdesc,$trails,$allitems,$codetitles) = @_;
     my $output;
     my %add_entries = (topmargin    => "0",
                        marginheight => "0",);
     my $js = '<script type="text/javascript">'."\n".
-             &courselink_javascript().'</script>'."\n";
+             &courselink_javascript($r).'</script>'."\n";
     my $start_page =
         &Apache::loncommon::start_page('Course/Community Catalog',$js,
                                        {'add_entries' => \%add_entries, });
@@ -409,9 +711,11 @@ END
 }
 
 sub courselink_javascript {
+    my ($r) = @_;
+    my $hostname = $r->hostname();    
     return <<"END";
 
-function ToSyllabus(cdom,cnum) {
+function ToSyllabus(cdom,cnum,usehttp) {
     if (cdom == '' || cdom == null) {
         return;
     }
@@ -419,6 +723,13 @@ function ToSyllabus(cdom,cnum) {
         return;
     }
     document.linklaunch.action = "/public/"+cdom+"/"+cnum+"/syllabus";
+    if (usehttp == 1) {
+        var hostname = '$hostname';
+        if (hostname != '') {
+            document.linklaunch.action = 'http://'+hostname+document.linklaunch.action;
+        }
+        document.linklaunch.action += '?usehttp=1';
+    }
     document.linklaunch.submit();
 }
 
@@ -436,13 +747,9 @@ END
 
 sub instcode_course_selector {
     my ($r,$codedom,$formname,$domdesc,$catlinks,$catjs,$codetitles,$cattype,$canviewall) = @_;
-    my %coursecodes = ();
-    my %codes = ();
     my %cat_titles = ();
     my %cat_order = ();
-    my %cat_items;
-    my $caller = 'global';
-    my $format_reply;
+    my %cat_items = ();
     my %add_entries = (topmargin    => "0",
                        marginheight => "0",);
     my ($jscript,$totcodes,$numtitles,$lasttitle) = 
@@ -1248,7 +1555,7 @@ sub build_courseinfo_hash {
         $cleandesc=~s/'/\\'/g;
         $cleandesc =~ s/^\s+//;
         my ($cdom,$cnum)=split(/\_/,$course);
-        my ($instcode,$singleowner,$ttype,$selfenroll_types,
+        my ($instcode,$singleowner,$ttype,$selfenroll_types,$extsyllplain,
             $selfenroll_start,$selfenroll_end,@owners,%ownernames,$categories);
         if (ref($courses->{$course}) eq 'HASH') {
             $descr = $courses->{$course}{'description'};
@@ -1259,6 +1566,7 @@ sub build_courseinfo_hash {
             $selfenroll_start = $courses->{$course}{'selfenroll_start_date'};
             $selfenroll_end = $courses->{$course}{'selfenroll_end_date'};
             $categories = $courses->{$course}{'categories'};
+            $extsyllplain = $courses->{$course}{'extsyllplain'};
             push(@owners,$singleowner);
             if ($courses->{$course}{'co-owners'} ne '') {
                 foreach my $item (split(/,/,$courses->{$course}{'co-owners'})) {
@@ -1302,6 +1610,7 @@ sub build_courseinfo_hash {
         $courseinfo{$course}{'selfenroll_start'} = $selfenroll_start;
         $courseinfo{$course}{'selfenroll_end'} = $selfenroll_end;
         $courseinfo{$course}{'categories'} = $categories;
+        $courseinfo{$course}{'extsyllplain'} = $extsyllplain;
 
         my %coursehash = &Apache::lonnet::dump('environment',$cdom,$cnum);
         my @classids;
@@ -1486,7 +1795,8 @@ sub count_students {
 sub courseinfo_row {
     my ($info,$knownuser,$details,$countref,$now,$course,$trails,$allitems,$numbers,$canviewall) = @_;
     my ($cdom,$cnum,$title,$ownerlast,$code,$owner,$seclist,$xlist_items,
-        $accessdates,$showsyllabus,$counts,$autoenrollment,$output,$categories);
+        $accessdates,$showsyllabus,$counts,$autoenrollment,$output,$categories,
+        $extsyllplain);
     if (ref($info) eq 'HASH') {
         $cdom = $info->{'cdom'};
         $cnum = $info->{'cnum'};
@@ -1500,6 +1810,7 @@ sub courseinfo_row {
         $counts = $info->{'counts'};
         $autoenrollment = $info->{'autoenrollment'};
         $showsyllabus = $info->{'showsyllabus'};
+        $extsyllplain = $info->{'extsyllplain'};
         $categories = $info->{'categories'};
     } else {
         $output = '<td colspan="8">'.&mt('No information available for [_1].',
@@ -1527,7 +1838,13 @@ sub courseinfo_row {
                '<td>'.$xlist_items.'</td>'.
                '<td>'.$title.'&nbsp;<font size="-2">';
     if ($showsyllabus) {
-        $output .= '<a href="javascript:ToSyllabus('."'$cdom','$cnum'".')">'.&mt('Syllabus').'</a>';
+        my $usehttp = 0;
+        if (($ENV{'SERVER_PORT'} == 443) && ($extsyllplain)) {
+            unless (&Apache::lonnet::uses_sts()) {
+                $usehttp = 1;
+            }
+        }
+        $output .= '<a href="javascript:ToSyllabus('."'$cdom','$cnum','$usehttp'".')">'.&mt('Syllabus').'</a>';
     } else {
         $output .= '&nbsp;';
     }
@@ -1607,27 +1924,32 @@ sub identify_sections {
 sub get_valid_classes {
     my ($seclist,$xlist_items,$crscode,$owners,$cdom,$cnum) = @_;
     my $response;
-    my %validations;
+    my (@sections,@xlists,%possclasses,%okclasses,%validations);
     @{$validations{'sections'}} = ();
     @{$validations{'xlists'}} = ();
     my $totalitems = 0;
     if ($seclist) {
-        foreach my $sec (split(/, /,$seclist)) {
-            my $class = $crscode.$sec;
-            if (&Apache::lonnet::auto_validate_class_sec($cdom,$cnum,$owners,
-							 $class) eq 'ok') {
+        @sections = split(/,\s+/,$seclist);
+        map { $possclasses{$crscode.$_} = 1; } @sections;
+    }
+    if ($xlist_items) {
+        @xlists = split(/,\s+/,$xlist_items);
+        map { $possclasses{$_} = 1; } @xlists;
+    }
+    my %okclasses = &Apache::lonnet::auto_validate_instclasses($cdom,$cnum,$owners,
+                                                               \%possclasses);
+    if (keys(%okclasses)) {
+        foreach my $sec (@sections) {
+            if ($okclasses{$crscode.$sec}) {
                 if (!grep(/^\Q$sec$\E/,@{$validations{'sections'}})) {
                     push(@{$validations{'sections'}},$sec);
                     $totalitems ++;
                 }
             }
         }
-    }
-    if ($xlist_items) {
-        foreach my $item (split(/, /,$xlist_items)) {
-            if (&Apache::lonnet::auto_validate_class_sec($cdom,$cnum,$owners,
-							 $item) eq 'ok') {
-                if (!grep(/^\Q$item$\E/,@{$validations{'xlists'}})) {
+        foreach my $item (@xlists) {
+            if ($okclasses{$item}) {
+                if (!grep(/^\Q$item\E$/,@{$validations{'xlists'}})) {
                     push(@{$validations{'xlists'}},$item);
                     $totalitems ++;
                 }
