@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Personal Information Page
 #
-# $Id: lonaboutme.pm,v 1.158 2014/12/01 22:52:48 raeburn Exp $
+# $Id: lonaboutme.pm,v 1.159 2020/12/18 15:23:02 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -671,7 +671,7 @@ sub portfolio_files {
 sub build_hierarchy {
     my ($r,$cdom,$cnum,$portaccess,$is_course,$filecounts,$mode,$access_info,
         $allfileshash,$group) = @_;
-    my $clientip = $r->get_remote_host();
+    my $clientip = &Apache::lonnet::get_requestor_ip($r);
     foreach my $filename (sort(keys(%{$access_info}))) {
         my $access_status =
            &Apache::lonnet::get_portfolio_access($cdom,$cnum,$filename,$group,$clientip,
