@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # displays the blocking status table
 #
-# $Id: lonblockingstatus.pm,v 1.13 2016/01/27 00:24:09 raeburn Exp $
+# $Id: lonblockingstatus.pm,v 1.14.2.4 2020/09/29 19:55:00 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -43,42 +43,47 @@ sub handler {
     $r->send_http_header;
     return OK if $r->header_only;
 
-    my (%activities,$activity,$origurl);
-    map { $activities{$_} = 1; } ('boards','chat','msgdisplay','blogs','groups','port','printout','docs','passwd');
+    my (%activities,$activity,$origurl,$origsymb);
+    map { $activities{$_} = 1; } ('boards','chat','com','blogs','groups','port','printout','docs','passwd');
 
     # determine what kind of blocking we want details for
-    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['activity','url']);
+    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['activity','url','symb']);
     $activity = $env{'form.activity'};
 
     my $title = 'Communication Blocking Status Information';
 
-    if (($activity eq 'docs') || ($activity eq 'printout')) {
+    if (($activity eq 'docs') || ($activity eq 'printout') ||
+        ($activity eq 'passwd')) {
         $title = 'Blocking Status Information';
-        $origurl = $env{'form.url'};
+        if ($activity eq 'docs') {
+            $origurl = $env{'form.url'};
+            $origsymb = $env{'form.symb'};
+        }
     }
     $r->print(&Apache::loncommon::start_page($title,undef,
                                             {'only_body' => 1}));
 
     if (($activity eq '') || (!$activities{$activity})) {
-        $r->print('<span class="LC_error">'.&mt('Error: unknown activity type blocked').'</span>');
+        $r->print('<p class="LC_error">'.&mt('Error: unknown activity type blocked').'</p>');
+    } elsif (($activity eq 'docs') && ($origurl eq '') && ($origsymb eq '')) {
+        $r->print('<p class="LC_error">'.&mt('Error: could not determine what content was blocked from access').'</p>');
     } else {
-        $r->print(&blockpage($activity,$origurl));
+        $r->print(&blockpage($activity,$origurl,$origsymb));
     }
-    
+
     $r->print(&Apache::loncommon::end_page());
 
     return OK;
 }
 
-
 sub blockpage {
-    my ($activity,$origurl) = @_;
+    my ($activity,$origurl,$origsymb) = @_;
 
     # in case of a portfolio block we need to determine the owner of the files
     # we're trying to look at. This information is passed via query string.
     my ($uname, $udom);
 
-    if (($activity eq 'port') || 
+    if (($activity eq 'port') ||
         (($activity eq 'passwd') && ($env{'user.name'} eq 'public') && ($env{'user.domain'} eq 'public'))) {
         &Apache::loncommon::get_unprocessed_cgi(
             $ENV{'QUERY_STRING'}, ['udom', 'uname'] );
@@ -89,7 +94,7 @@ sub blockpage {
                 return '<span class="LC_error">'.
                        &mt('Information about the owner of the portfolio files you were trying to view was missing or invalid.').
                        '</span><br />'.
-                       &mt('Without valid owner information, the reason why access is blocked can not be determined'); 
+                       &mt('Without valid owner information, the reason why access is blocked can not be determined');
             } else {
                 return '<span class="LC_error">'.
                        &mt('Information about the username and/or domain for which you were trying to reset a password was missing or invalid.').
@@ -99,33 +104,63 @@ sub blockpage {
         }
     }
 
-
     # retrieve start/end of possible active blocking
-    my %setters;
-    my ($startblock,$endblock,$triggerblock) = 
-        &Apache::loncommon::blockcheck(\%setters,$activity,$uname,$udom,$origurl);
+    my (%setters,$startblock,$endblock,$triggerblock);
+
+    if ($activity eq 'docs') {
+        my ($cdom,$cnum);
+        if ($env{'request.course.id'}) {
+            $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+            $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+        }
+        my $cancheck;
+        if (($cdom ne '') && ($cnum ne '')) {
+            if ($origsymb =~ m{^uploaded/($match_domain)/($match_courseid)/}) {
+                my ($crsdom,$crsnum) = ($1,$2);
+                if (($cdom eq $crsdom) && ($cnum eq $crsnum)) {
+                    $cancheck = 1;
+                }
+            } else {
+                $cancheck = 1;
+            }
+        }
+        if ($cancheck) {
+            ($startblock,$endblock,$triggerblock) =
+                &Apache::loncommon::blockcheck(\%setters,$activity,$cnum,$cdom,$origurl,1,$origsymb,'blockingstatus');
+        } else {
+            return '<p class="LC_info">'.&mt('Could not determine why access is blocked.').'</p>';
+        }
+    } else {
+        ($startblock,$endblock,$triggerblock) =
+            &Apache::loncommon::blockcheck(\%setters,$activity,$uname,$udom,$origurl,undef,$origsymb,'blockingstatus');
+    }
 
     # nothing to do if there's no active blocking
-    unless ($startblock && $endblock) { return ''; }
+    unless ($startblock && $endblock) {
+        if ($activity eq 'docs') {
+            return '<p class="LC_info">'.&mt('Content no longer blocked from access').'</p>';
+        }
+        return '<p class="LC_info">'.&mt('Access no longer blocked for this activity').'</p>';
+    }
 
     # lookup $activity -> description
                    #possible activity          #corresponding description
     my %descs = (
                    boards     => 'Discussion posts in this course',
                    chat       => 'Chat Room',
-                   msgdisplay => 'This message',
+                   com        => 'This message',
                    blogs      => 'Blogs',
                    groups     => 'Groups in this course',
-                   printout   => 'Printout generation', 
+                   printout   => 'Printout generation',
                    docs       => 'Course Content',
                    passwd     => 'Changing of passwords',
                 );
 
     if ($activity eq 'groups' || $activity eq 'boards') {
         if (&Apache::loncommon::course_type() eq 'Community') {
-            $descs{'boards'} = 'Discussion posts in this community',
-            $descs{'groups'} = 'Groups in this community',
-            $descs{'docs'} = 'Community Content',  
+            $descs{'boards'} = 'Discussion posts in this community';
+            $descs{'groups'} = 'Groups in this community';
+            $descs{'docs'} = 'Community Content';
         }
     }
 
@@ -141,11 +176,11 @@ sub blockpage {
     my $showend   = Apache::lonlocal::locallocaltime($endblock);
 
     my $output;
-    
+
     if ( ref($description) ne 'ARRAY' ) {
         #default: $description is one of the above descriptions
         if ($activity eq 'docs') {
-            $output=&mt( 'Access to the content page you are attempting to' 
+            $output=&mt( 'Access to the content page you are attempting to'
                          . ' view will be unavailable between [_1] and [_2] because'
                          . ' access to selected '.$description.' is being blocked.'
                          ,$showstart, $showend);
@@ -158,7 +193,7 @@ sub blockpage {
             $output = mt( $description
                           . ' will be inaccessible between [_1] and [_2] because'
                           . ' communication is being blocked.'
-                          ,$showstart, $showend);  
+                          ,$showstart, $showend);
         }
     } else {
         # @$description is is the array returned from get_portfolio_category()
@@ -167,12 +202,12 @@ sub blockpage {
         $output = mt( $$description[0]
                       . ' will be inaccessible between [_2] and [_3] because'
                       . ' communication is being blocked.'
-                     ,$$description[1], $showstart, $showend)  
+                     ,$$description[1], $showstart, $showend);
     }
 
     $output = "<p class=\"LC_info\">$output</p>";
 
-    # show a table containing details, except if user is trying to look 
+    # show a table containing details, except if user is trying to look
     # at a different user's portfolio files
     if (   $activity ne 'port'                        # no portfolio
         || (   $uname eq $env{'user.name'}            # or own portfolio
@@ -213,7 +248,7 @@ sub build_block_table {
             }
             my $triggertype = $$setters{$course}{triggers}[$i];
             if ($triggertype) {
-                $fullname .= &mt(' (triggered by you when starting timer)'); 
+                $fullname .= &mt(' (triggered by you when starting timer)');
             }
             my ($openblock,$closeblock) = @{$$setters{$course}{times}[$i]};
             $openblock = &Apache::lonlocal::locallocaltime($openblock);
