@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Handler to set domain-wide configuration settings
 #
-# $Id: lonconfigsettings.pm,v 1.21.4.9 2016/10/23 21:13:51 raeburn Exp $
+# $Id: lonconfigsettings.pm,v 1.21.4.11 2019/08/27 20:43:52 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -39,7 +39,7 @@ use Apache::courseclassifier();
 use LONCAPA qw(:DEFAULT :match);
 
 sub print_header {
-    my ($r,$phase,$context,$jscript,$container) = @_;
+    my ($r,$phase,$context,$jscript,$container,$instcode,$dom) = @_;
     my ($pagetitle,$brcrumtitle,$action,$call_category_check,$instcode_check,
         $crstype,@actions,@code_order);
     if ($phase eq 'display') {
@@ -176,11 +176,50 @@ $jscript
         $additem = {'add_entries' => \%loaditems,};
     } elsif ($phase eq 'display') {
         if ($context eq 'domain') {
+            my $onload;
             if (grep(/^coursedefaults$/,@actions)) {
+                $onload = "toggleDisplay(document.display,'cloneinstcode');".
+                          "toggleDisplay(document.display,'credits');".
+                          "toggleDisplay(document.display,'studentsubmission');";
+            }
+            if (grep(/^selfcreation$/,@actions)) {
+                my $prefix = 'cancreate_emailverified';
+                my $customclass = 'LC_selfcreate_email';
+                my $classprefix = 'LC_canmodify_emailusername_';
+                my $optionsprefix = 'LC_options_emailusername_';
+                $onload .= "toggleRows(document.display,'cancreate_email','selfassign','$customclass','$classprefix','$optionsprefix');";
+                my ($othertitle,$usertypes,$types) = &Apache::loncommon::sorted_inst_types($dom);
+                my $hascustom;
+                my ($emailrules,$emailruleorder) = &Apache::lonnet::inst_userrules($dom,'email');
+                if (ref($emailrules) eq 'HASH') {
+                    if (keys(%{$emailrules}) > 0) {
+                        $hascustom = 'cancreate_emailrule';
+                    }
+                }
+                my @posstypes;
+                if (ref($types) eq 'ARRAY') {
+                    @posstypes = @{$types};
+                    push(@posstypes,'default');
+                    foreach my $type (@posstypes) {
+                        $onload .= "toggleEmailOptions(document.display,'cancreate_emailoptions','$hascustom',".
+                                                               "'cancreate_emaildomain','$type');";
+                    }
+                } else {
+                    $onload .= "toggleEmailOptions(document.display,'cancreate_emailoptions','$hascustom',".
+                                                       "'cancreate_emaildomain','default');";
+                }
+            }
+            if (grep(/^contacts$/,@actions)) {
+                my $customclass = 'LC_helpdesk_override';
+                my $optionsprefix = 'LC_options_helpdesk_';
+                $onload .= "toggleHelpdeskRow(document.display,'overrides','$customclass','$optionsprefix');";
+            }
+            if (grep(/^scantron$/,@actions)) {
+                $onload .= "toggleScantron('document.display');";
+            }
+            if ($onload) {
                 my %loaditems = (
-                    '             onload' => "toggleDisplay(document.display,'cloneinstcode');".
-                                             "toggleDisplay(document.display,'credits');".
-                                             "toggleDisplay(document.display,'studentsubmission');",
+                                  'onload' => $onload,
                                 );
                 $additem = {'add_entries' => \%loaditems,};
             }
@@ -335,7 +374,7 @@ sub display_settings {
     if (ref($values) eq 'HASH') {
         $instcode = $values->{'internal.coursecode'};
     }
-    &print_header($r,$phase,$context,$jscript,$container,$instcode);
+    &print_header($r,$phase,$context,$jscript,$container,$instcode,$dom);
     my $divwidth = 900;
     if ((ref($prefs_order) eq 'ARRAY') && (ref($prefs) eq 'HASH') && (ref($values) eq 'HASH')) { 
         if (@actions > 0) {
