@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Sending messages
 #
-# $Id: lonnotify.pm,v 1.42 2017/01/18 21:07:31 raeburn Exp $
+# $Id: lonnotify.pm,v 1.44 2021/03/06 13:41:46 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -378,7 +378,6 @@ ENDSCRIPT
 
 sub print_selection_form {
     my ($r,$formname,$cdom) = @_;
-    my %coursecodes = ();
     my %codes = ();
     my @codetitles = ();
     my %cat_titles = ();
@@ -386,9 +385,7 @@ sub print_selection_form {
     my %idlist = ();
     my %idnums = ();
     my %idlist_titles = ();
-    my $caller = 'global';
     my $totcodes = 0;
-    my $format_reply;
     my $jscript = '';
     my %lt=&Apache::lonlocal::texthash(
                'buil' => 'Building valid e-mail address from username, if missing from preferences:',
@@ -399,10 +396,15 @@ sub print_selection_form {
     &Apache::lonhtmlcommon::add_breadcrumb
           ({text=>"Select Audience"});
 
-    $totcodes = &Apache::courseclassifier::retrieve_instcodes(\%coursecodes,$cdom,$totcodes);
-    if ($totcodes > 0) {
-        $format_reply = &Apache::lonnet::auto_instcode_format($caller,$cdom,\%coursecodes,\%codes,\@codetitles,\%cat_titles,\%cat_order);
-        if ($format_reply eq 'ok') {
+    my $instcats = &Apache::lonnet::get_dom_instcats($cdom);
+    if (ref($instcats) eq 'HASH') {
+        if ((ref($instcats->{'codetitles'}) eq 'ARRAY') && (ref($instcats->{'codes'}) eq 'HASH') &&
+            (ref($instcats->{'cat_titles'}) eq 'HASH') && (ref($instcats->{'cat_order'}) eq 'HASH')) {
+            %codes = %{$instcats->{'codes'}};
+            @codetitles = @{$instcats->{'codetitles'}};
+            %cat_titles = %{$instcats->{'cat_titles'}};
+            %cat_order = %{$instcats->{'cat_order'}};
+            $totcodes = scalar(keys(%codes));
             my $numtypes = @codetitles;
             &Apache::courseclassifier::build_code_selections(\%codes,\@codetitles,\%cat_titles,\%cat_order,\%idlist,\%idnums,\%idlist_titles);
             my ($scripttext,$longtitles) = &Apache::courseclassifier::javascript_definitions(\@codetitles,\%idlist,\%idlist_titles,\%idnums,\%cat_titles);
@@ -786,7 +788,7 @@ sub form_elements {
                    previous => &mt('Previously had access'),
                    future => &mt('Will have future access'),
                    );
-   %{$authtypes} = (
+   %{$authtypes} = &Apache::lonlocal::texthash (
                    krb4 => 'Kerberos 4',
                    krb5 => 'Kerberos 5',
                    internal => 'Internal (LON-CAPA)',
