@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Quick Student Grades Display
 #
-# $Id: lonquickgrades.pm,v 1.49.6.3 2017/04/02 13:42:06 raeburn Exp $
+# $Id: lonquickgrades.pm,v 1.49.6.7 2021/03/04 01:42:20 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -58,7 +58,13 @@ sub real_handler {
         return OK;
     }
 
-    unless (&Apache::lonnet::allowed('mgr')) {
+    my $cangrade=&Apache::lonnet::allowed('mgr');
+    my $showPoints =
+        $env{'course.'.$env{'request.course.id'}.'.grading'} eq 'standard';
+
+    my $reinitresult;
+
+    unless ($cangrade) {
         # Check for critical messages and redirect if present.
         my ($redirect,$url) = &Apache::loncommon::critical_redirect(300);
         if ($redirect) {
@@ -69,78 +75,141 @@ sub real_handler {
 
         # Check if course needs to be re-initialized
         my $loncaparev = $r->dir_config('lonVersion');
-        my ($result,@reinit) = &Apache::loncommon::needs_coursereinit($loncaparev);
+        ($reinitresult,my @reinit) = &Apache::loncommon::needs_coursereinit($loncaparev);
 
-        if ($result eq 'switch') {
+        if ($reinitresult eq 'switch') {
             &Apache::loncommon::content_type($r,'text/html');
             $r->send_http_header;
             $r->print(&Apache::loncommon::check_release_result(@reinit));
             return OK;
-        } elsif ($result eq 'update') {
-            my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
-            my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+        } elsif ($reinitresult eq 'update') {
+            my $cid = $env{'request.course.id'};
+            my $cnum = $env{'course.'.$cid.'.num'};
+            my $cdom = $env{'course.'.$cid.'.domain'};
+            &Apache::loncommon::content_type($r,'text/html');
+            $r->send_http_header;
+            &startpage($r,$showPoints);
+            my $preamble = '<div id="LC_update_'.$cid.'" class="LC_info">'.
+                           '<br />'.
+                           &mt('Your course session is being updated because of recent changes by course personnel.').
+                           ' '.&mt('Please be patient').'.<br /></div>'.
+                           '<div style="padding:0;clear:both;margin:0;border:0"></div>';
+            my %prog_state = &Apache::lonhtmlcommon::Create_PrgWin($r,undef,$preamble);
+            &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,&mt('Updating course'));
+            $r->rflush();
             my ($furl,$ferr) = &Apache::lonuserstate::readmap("$cdom/$cnum");
+            &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,&mt('Finished!'));
+            &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
+            my $closure = <<ENDCLOSE;
+<script type="text/javascript">
+// <![CDATA[
+\$("#LC_update_$cid").hide('slow');
+// ]]>
+</script>
+ENDCLOSE
             if ($ferr) {
+                $r->print($closure.&Apache::loncommon::end_page());
                 my $requrl = $r->uri;
                 $env{'user.error.msg'}="$requrl:bre:0:0:Course not initialized";
                 $env{'user.reinit'} = 1;
                 return HTTP_NOT_ACCEPTABLE;
+            } else {
+               $r->print($closure);
             }
         }
     }
 
-    # Send header, don't cache this page
-    &Apache::loncommon::no_cache($r);
-    $r->send_http_header;
+    unless ($reinitresult eq 'update') {
+        # Send header, don't cache this page
+        &Apache::loncommon::no_cache($r);
+        $r->send_http_header;
+        &startpage($r,$showPoints);
+    }
+    $r->rflush();
+
+    &startGradeScreen($r);
+
+#
+# Pick student
+#
+    my $uname;
+    my $udom;
+    my $stdid;
+    if ($cangrade) {
+        if ($env{'form.uname'}) { $uname=$env{'form.uname'}; }
+        if ($env{'form.udom'}) { $udom=$env{'form.udom'}; }
+        if ($env{'form.id'}) { $stdid=$env{'form.id'}; }
+        if (($stdid) && ($udom)) {
+            $uname=(&Apache::lonnet::idget($udom,[$stdid],'ids'))[1];
+        }
+        if (($stdid) && (!$uname)) {
+            $r->print('<p><span class="LC_warning">'.&mt("Unknown Student/Employee ID: [_1]",$stdid).'</span></p>');
+            $stdid='';
+        }
+        if (($uname eq '') && ($udom eq '')) {
+            $uname = $env{'user.name'};
+            $udom = $env{'user.domain'};
+        }
+        $r->print('<form method="post" name="quickform" action="/adm/quickgrades">');
+        my $chooseopt=&Apache::loncommon::select_dom_form($udom,'udom').' '.
+           &Apache::loncommon::selectstudent_link('quickform','uname','udom');
+        $r->print("<p>\n".&Apache::loncommon::studentbrowser_javascript()."\n");
+        $r->print(&mt('For User [_1] or Student/Employee ID [_2] at Domain [_3]'
+                 ,'<input type="text" value="'.$uname.'" size="12" name="uname" />'
+                 ,'<input type="text" value="'.$stdid.'" size="12" name="id" /> '
+                 ,$chooseopt).
+                 '&nbsp;&nbsp;<input type="submit" name="display" value="'.&mt('Display Individual Student').'" /></p>');
+        if (($uname) && ($udom)) {
+            $r->print('<p>'.&mt('Full Name: [_1]',&Apache::loncommon::plainname($uname,$udom)).'</p>');
+        }
+    } else {
+        $r->print('<p class="LC_info">'.&mt('This may take a few moments to display.').'</p>');
+    }
+    $r->rflush();
 
-    my $showPoints = 
-        $env{'course.'.$env{'request.course.id'}.'.grading'} eq 'standard';
-    my $notshowSPRSlink = 
-        (($env{'course.'.$env{'request.course.id'}.'.grading'} eq 'external')
-      || ($env{'course.'.$env{'request.course.id'}.'.grading'} eq 'externalnototals'));
     my $notshowTotals=
         $env{'course.'.$env{'request.course.id'}.'.grading'} eq 'externalnototals';
 
-    # Create the nav map
-    my $navmap = Apache::lonnavmaps::navmap->new();
+    my ($navmap,$totalParts,$totalPossible,$totalRight,$totalAttempted,$topLevelParts,
+        $topLevelRight,$topLevelAttempted) = &getData($showPoints,$uname,$udom);
 
-    if (!defined($navmap)) {
+    if (ref($navmap)) {
+        &outputTable($r,$showPoints,$notshowTotals,$navmap,$totalParts,$totalPossible,
+                     $totalRight,$totalAttempted,$topLevelParts,$topLevelRight,
+                     $topLevelAttempted);
+    } else {
+        if ($cangrade) { $r->print("\n</form>\n"); }
         my $requrl = $r->uri;
-        $env{'user.error.msg'} = "$requrl:bre:0:0:Navamp initialization failed.";
+        $env{'user.error.msg'} = "$requrl:bre:0:0:Navmap initialization failed.";
         return HTTP_NOT_ACCEPTABLE;
     }
+    if ($cangrade) { $r->print("\n</form>\n"); }
+    &endGradeScreen($r);
+    return OK;
+}
 
-    # Keep this hash in sync with %statusIconMap in lonnavmaps; they
-    # should match color/icon
-    my $res = $navmap->firstResource(); # temp resource to access constants
- 
-    # Header
-    my $title = "Grading and Statistics";#$showPoints ? "Points Display" : "Completed Problems Display";
-    my $brcrum = [{href=>"/adm/quickgrades",text => "Points Display"}];
-    $r->print(&Apache::loncommon::start_page($title,undef,
-                                            {'bread_crumbs' => $brcrum})
-             );
+#
+# Go through the complete course and collect data
+#
 
-    if (!$showPoints && !$notshowSPRSlink ) {
-        $r->print('<p>'
-                 .&mt('This screen shows how many problems (or problem parts) you have completed'
-                     .', and how many you have not yet done.'
-                     .' You can also look at [_1]a detailed score sheet[_2].'
-                     ,'<a href="/adm/studentcalc">','</a>')
-                 .'</p>');
+sub getData {
+
+    my ($showPoints,$uname,$udom)=@_;
+
+    # Create the nav map
+    my $navmap = Apache::lonnavmaps::navmap->new($uname,$udom);
+
+    if (!defined($navmap)) {
+        return ();
     }
 
-    $r->print('<p class="LC_info">'.&mt('This may take a few moments to display.').'</p>');
-
-    $r->rflush();
-
-    # End navmap using boilerplate
+    my $res = $navmap->firstResource(); # temp resource to access constants
 
     my $iterator = $navmap->getIterator(undef, undef, undef, 1);
     my $depth = 1;
     $iterator->next(); # ignore first BEGIN_MAP
     my $curRes = $iterator->next();
-    
+
     # General overview of the following: Walk along the course resources.
     # For every problem in the resource, tell its parent maps how many
     # parts and how many parts correct it has. After that, each map will
@@ -173,35 +242,36 @@ sub real_handler {
             my $stack = $iterator->getStack();
             
             for my $part (@{$parts}) {
-		my $completionStatus = $curRes->getCompletionStatus($part);
 		my $dateStatus = $curRes->getDateStatus($part);
-		
-                if ($completionStatus == $curRes->EXCUSED()) {
+                my $weight = $curRes->weight($part);
+                my $problemstatus = $curRes->problemstatus($part);
+
+                if ($curRes->solved($part) eq 'excused') {
                     next;
                 }
 		if ($showPoints) {
 		    my $score = 0;
 		    # If we're not telling status and the answer date isn't passed yet, 
 		    # it's an "attempted" point
-		    if ((($curRes->problemstatus($part) eq 'no') ||
-                        ($curRes->problemstatus($part) eq 'no_feedback_ever')) &&
+		    if ((($problemstatus eq 'no') ||
+                         ($problemstatus eq 'no_feedback_ever')) &&
 			($dateStatus != $curRes->ANSWER_OPEN)) {
 			my $status = $curRes->simpleStatus($part);
 			if ($status == $curRes->ATTEMPTED) {
-			    $partsAttempted += $curRes->weight($part);
+			    $partsAttempted += $weight;
 			    $totalAttempted += $partsAttempted;
 			}
 		    } else {
-			$score = &Apache::grades::compute_points($curRes->weight($part), $curRes->awarded($part));
+			$score = &Apache::grades::compute_points($weight, $curRes->awarded($part));
 		    }
 		    $partsRight += $score;
 		    $totalRight += $score;
-		    $partsCount += $curRes->weight($part);
+		    $partsCount += $weight;
 
 		    if ($curRes->opendate($part) < $now) {
-			$totalPossible += $curRes->weight($part);
+			$totalPossible += $weight;
 		    }
-		    $totalParts += $curRes->weight($part);
+		    $totalParts += $weight;
 		} else {
 		    my $status = $curRes->simpleStatus($part);
 		    my $thisright = 0;
@@ -218,7 +288,6 @@ sub real_handler {
 			$totalAttempted++;
 		    }
 		    
-		    my $dateStatus = $curRes->getDateStatus($part);
 		    $totalParts++;
 		    if ($curRes->opendate($part) < $now) {
 			$totalPossible++;
@@ -249,11 +318,18 @@ sub real_handler {
         }
         $curRes = $iterator->next();
     }
+    return ($navmap,$totalParts,$totalPossible,$totalRight,$totalAttempted,
+            $topLevelParts,$topLevelRight,$topLevelAttempted);
+}
 
-    $iterator = $navmap->getIterator(undef, undef, undef, 1);
-    $depth = 1;
-    $iterator->next(); # ignore first BEGIN_MAP
-    $curRes = $iterator->next();
+#
+# Outputting everything.
+#
+
+sub outputTable {
+
+    my ($r,$showPoints,$notshowTotals,$navmap,$totalParts,$totalPossible,$totalRight,
+        $totalAttempted,$topLevelParts,$topLevelRight,$topLevelAttempted)=@_;
 
     my @start = (255, 255, 192);
     my @end   = (0, 192, 0);
@@ -264,12 +340,21 @@ sub real_handler {
     $r->print(&Apache::loncommon::start_data_table()
              .&Apache::loncommon::start_data_table_header_row()
              .'<th>'.&mt('Folder').'</th>');
-    $title = &mt($showPoints ? "Points Scored" : "Done");
+    my $title = &mt($showPoints ? "Points Scored" : "Done");
     if ($totalAttempted) {
-	$title .= " / " . &mt("Attempted");
+        $title .= " / " . &mt("Attempted");
     }
     $r->print("<th>$title".($notshowTotals?'':" / ".&mt('Total')).'</th>'
              .&Apache::loncommon::end_data_table_header_row());
+#
+# Output of folder scores
+#
+
+    my $iterator = $navmap->getIterator(undef, undef, undef, 1);
+    my $depth = 1;
+    $iterator->next(); # ignore first BEGIN_MAP
+    my $curRes = $iterator->next();
+
     while ($depth > 0) {
         if ($curRes == $iterator->BEGIN_MAP()) {$depth++;}
         if ($curRes == $iterator->END_MAP()) { $depth--; }
@@ -284,7 +369,7 @@ sub real_handler {
             if ($total > 0) {
                 my $ratio;
                 $ratio = $correct / $total;
-                my $color = mixColors(\@start, \@end, $ratio);
+                my $color = &mixColors(\@start, \@end, $ratio);
                 $r->print(&Apache::loncommon::start_data_table_row()
                          .'<td style="background-color:'.$color.';">');
                 
@@ -317,7 +402,7 @@ sub real_handler {
     # If there were any problems at the top level, print an extra "catchall"
     if ($topLevelParts > 0) {
         my $ratio = $topLevelRight / $topLevelParts;
-        my $color = mixColors(\@start, \@end, $ratio);
+        my $color = &mixColors(\@start, \@end, $ratio);
         $r->print(&Apache::loncommon::start_data_table_row()
                  .'<td style="background-color:'.$color.';">');
         $r->print(&mt("Problems Not Contained In A Folder")."</td><td>");
@@ -325,8 +410,11 @@ sub real_handler {
                  .&Apache::loncommon::end_data_table_row());
     }
 
+#
+# show totals (if applicable), close table
+#
     if ($showPoints) {
-	my $maxHelpLink = Apache::loncommon::help_open_topic("Quick_Grades_Possibly_Correct");
+	my $maxHelpLink = &Apache::loncommon::help_open_topic("Quick_Grades_Possibly_Correct");
 
 	$title = $showPoints ? "Points" : "Parts Done";
 	my $totaltitle = $showPoints ? &mt("Awarded Total Points") : &mt("Total Parts Done");
@@ -338,10 +426,68 @@ sub real_handler {
                  .&Apache::loncommon::end_data_table_row());
     }
 
-    $r->print(&Apache::loncommon::end_data_table()
-             .&Apache::loncommon::end_page());
+    $r->print(&Apache::loncommon::end_data_table());
+    return;
+}
 
-    return OK;
+sub startpage {
+    my ($r,$showPoints) = @_;
+    my $title = "Grading and Statistics";#$showPoints ? "Points Display" : "Completed Problems Display";
+    my $brcrum = [{href=>"/adm/quickgrades",text => "Points Display"}];
+    $r->print(&Apache::loncommon::start_page($title,undef,
+                                            {'bread_crumbs' => $brcrum})
+             );
+}
+
+sub startGradeScreen {
+    my ($r)=@_;
+
+    my $showPoints =
+        $env{'course.'.$env{'request.course.id'}.'.grading'} eq 'standard';
+    my $notshowSPRSlink =
+        (($env{'course.'.$env{'request.course.id'}.'.grading'} eq 'external')
+      || ($env{'course.'.$env{'request.course.id'}.'.grading'} eq 'externalnototals'));
+    my $notshowTotals =
+        $env{'course.'.$env{'request.course.id'}.'.grading'} eq 'externalnototals';
+    my $showSPRSlink =
+        $env{'course.'.$env{'request.course.id'}.'.grading'} eq 'spreadsheet';
+
+    my $allowed_to_view = &Apache::lonnet::allowed('vgr',$env{'request.course.id'});
+    if ((!$allowed_to_view) && ($env{'request.course.sec'} ne '')) {
+        $allowed_to_view = &Apache::lonnet::allowed('vgr',
+                               "$env{'request.course.id'}/$env{'request.course.sec'}");
+    }
+
+    my $allowed_to_edit = &Apache::lonnet::allowed('mgr',$env{'request.course.id'});
+    if ((!$allowed_to_edit) && ($env{'request.course.sec'} ne '')) {
+        $allowed_to_edit = &Apache::lonnet::allowed('mgr',
+                               "$env{'request.course.id'}/$env{'request.course.sec'}");
+    }
+
+    if ($allowed_to_view) {
+        my @notes;
+        push(@notes,&mt('Students do not see total points.')) if ($notshowTotals);
+        push(@notes,&mt('Students do not see link to spreadsheet.')) if ($notshowSPRSlink);
+        push(@notes,&mt('Students will see points based on problem weights.')) if ($showPoints);
+        push(@notes,&mt('Students will see link to spreadsheet.')) if ($showSPRSlink);
+        push(@notes,&Apache::lonhtmlcommon::coursepreflink(&mt('Grade display settings'),'grading'));
+        $r->print(&Apache::loncommon::head_subbox(join('&nbsp;&nbsp;',@notes)));
+    } elsif (!$allowed_to_edit) {
+        if (!$showPoints && !$notshowSPRSlink ) {
+            $r->print(&Apache::loncommon::head_subbox(
+                      &mt('This screen shows how many problems (or problem parts) you have completed'
+                     .', and how many you have not yet done.'
+                     .' You can also look at [_1]a detailed score sheet[_2].'
+                     ,'<a href="/adm/studentcalc">','</a>')));
+        }
+    }
+    return;
+}
+
+sub endGradeScreen {
+    my ($r)=@_;
+    $r->print(&Apache::loncommon::end_page());
+    return;
 }
 
 # Pass this two refs to arrays for the start and end color, and a number
