@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # RSS Feeder
 #
-# $Id: lonrss.pm,v 1.53.2.2 2017/01/21 22:36:45 raeburn Exp $
+# $Id: lonrss.pm,v 1.53.2.4 2019/08/14 16:06:42 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -345,8 +345,10 @@ sub handler {
 #	my $title = $displayfeedname?$displayfeedname:"Available RSS Feeds and Blogs";
         my $title = "My Space";
 	my $rss_link = &Apache::lonrss::rss_link($uname,$udom);
+        my $head_extra = $rss_link.'<script type="text/javascript" '
+                                  .'src="/res/adm/includes/file_upload.js"></script>';
 	my $brcrumb = [{href=>$rss_link,text=>"Available RSS Feeds and Blogs"}];
-	$r->print(&Apache::loncommon::start_page($title,$rss_link,
+	$r->print(&Apache::loncommon::start_page($title,$head_extra,
 			 {'bread_crumbs'   => $brcrumb,
 			  'domain'         => $udom,
 			  'force_register' => $env{'form.register'}}).
@@ -538,7 +540,19 @@ sub handler {
                         my $uploadlink;
                         if ($entry==$newid) {
 # Generate upload link only for last (new) entry
-			    $uploadlink=&Apache::inputtags::file_selector(0,0,'*','both');
+# Calculate the quota space available in the user's portfolio
+                            my $disk_quota = &Apache::loncommon::get_user_quota($env{'user.name'},
+                                                 $env{'user.domain'}); # expressed in MB
+                            my $portfolio_root = '/userfiles/portfolio';
+                            my $getpropath = 1;
+                            my $current_disk_usage = &Apache::lonnet::diskusage(
+                                $env{'user.domain'}, $env{'user.name'},
+                                $portfolio_root, $getpropath); # Expressed in kB
+                            # Convert to MB for use in file_selector()
+                            my $free_space = $disk_quota - ($current_disk_usage / 1024.);
+                            # Format this number since it will be displayed onscreen
+                            $free_space = sprintf("%.1f", $free_space);
+                            $uploadlink=&Apache::inputtags::file_selector(0,0,'*','both','',$free_space);
 			} else {
 # Otherwise, display
                             $uploadlink='<tt>'.$newsfeed{$id.'_enclosureurl'}.'</tt>'.
