@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Helpdesk request form
 #
-# $Id: lonsupportreq.pm,v 1.79.2.3 2017/01/23 19:57:30 raeburn Exp $
+# $Id: lonsupportreq.pm,v 1.79.2.8 2021/01/04 03:46:06 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -69,7 +69,9 @@ sub handler {
         }
     }
     my $origurl = $env{'form.origurl'};
-    $origurl =~ s{^https?://}{};
+    if ($origurl =~ m{^https?://[^/]+(.*)$}) {
+        $origurl =~ $1;
+    }
     $origurl =~ s/(`)//g;
     $origurl =~ s/\$/\(\$\)/g;
     my $command = $env{'form.command'};
@@ -151,7 +153,6 @@ sub print_request_form {
         }
     }
 
-    my %coursecodes;
     my %codes;
     my @codetitles;
     my %cat_titles;
@@ -159,9 +160,7 @@ sub print_request_form {
     my %idlist;
     my %idnums;
     my %idlist_titles;
-    my $caller = 'global';
     my $totcodes = 0;
-    my $format_reply;
     my $jscript = '';
     my $loaditems = qq|
 function initialize_codes() {
@@ -169,10 +168,17 @@ function initialize_codes() {
 }
     |;
     unless ($helpform{'course'} eq 'no') {
-        $totcodes = &Apache::courseclassifier::retrieve_instcodes(\%coursecodes,$codedom);
-        if ($totcodes > 0) {
-            $format_reply = &Apache::lonnet::auto_instcode_format($caller,$codedom,\%coursecodes,\%codes,\@codetitles,\%cat_titles,\%cat_order);
-            if ($format_reply eq 'ok') {
+        my $instcats = &Apache::lonnet::get_dom_instcats($codedom);
+        if (ref($instcats) eq 'HASH') {
+            if ((ref($instcats->{'codetitles'}) eq 'ARRAY') && (ref($instcats->{'codes'}) eq 'HASH') &&
+                (ref($instcats->{'cat_titles'}) eq 'HASH') && (ref($instcats->{'cat_order'}) eq 'HASH')) {
+                %codes = %{$instcats->{'codes'}};
+                @codetitles = @{$instcats->{'codetitles'}};
+                %cat_titles = %{$instcats->{'cat_titles'}};
+                %cat_order = %{$instcats->{'cat_order'}};
+                $totcodes = scalar(keys(%codes));
+            }
+            if ($totcodes > 0) {
                 my $numtypes = @codetitles;
                 &Apache::courseclassifier::build_code_selections(\%codes,\@codetitles,\%cat_titles,\%cat_order,\%idlist,\%idnums,\%idlist_titles);
                 my ($scripttext,$longtitles) = &Apache::courseclassifier::javascript_definitions(\@codetitles,\%idlist,\%idlist_titles,\%idnums,\%cat_titles);
@@ -316,7 +322,9 @@ $loaditems
 // ]]>
 </script>
 ENDJS
-    if ($recaptcha_version >=2) {
+    if ($knownuser) {
+        $js .="\n".'<script type="text/javascript" src="/res/adm/includes/file_upload.js"></script>';
+    } elsif ($recaptcha_version >=2) {
         $js.= "\n".'<script src="https://www.google.com/recaptcha/api.js"></script>'."\n";
     }
     my %add_entries = (
@@ -574,10 +582,11 @@ ENDJS
                     $showmax = $helpform{'maxsize'};
                 }
                 $showmax = ' ('.sprintf("%.2f",$showmax).' '.&mt('MB max.').')';
-                $output .= &Apache::lonhtmlcommon::row_title($html_lt{'opfi'},undef,$css[$i]).
-                       ' <input type="file" name="screenshot" size="20" /><br />'.
-                       "\n".$html_lt{'uplf'}.$showmax."\n".
-                       &Apache::lonhtmlcommon::row_closure();
+                $output .= &Apache::lonhtmlcommon::row_title($html_lt{'opfi'},undef,$css[$i])
+                    .' <input type="file" name="screenshot" class="LC_flUpload" size="20" />'
+                    .'<input type="hidden" id="LC_free_space" value="'.$max.'" />'
+                    .'<br />'."\n".$html_lt{'uplf'}.$showmax."\n"
+                    .&Apache::lonhtmlcommon::row_closure();
                 $num ++;
                 $i = $num%2;
             }
@@ -654,17 +663,20 @@ sub print_request_receipt {
             return;
         }
     }
-    my @ENVvars = ('HTTP_HOST','HTTP_USER_AGENT','REMOTE_ADDR','SERVER_ADDR','SERVER_NAME');
+    my $ip = &Apache::lonnet::get_requestor_ip();
+    my @ENVvars = ('HTTP_HOST','HTTP_USER_AGENT','SERVER_ADDR','SERVER_NAME');
     my @envvars = ('browser.os','browser.type','browser.version','user.home','request.role');
     my @loncvars = ('user.name','user.domain','request.course.sec','request.course.id');
-    my @cookievars = ('lonID');
+    my @cookievars;
+    if ($ENV{'SERVER_PORT'} == 443) {
+        @cookievars = ('lonLinkID');
+    } else {
+        @cookievars = ('lonID');
+    }
 
     my $admin = $Apache::lonnet::perlvar{'lonAdminMail'};
     my $origmail = $Apache::lonnet::perlvar{'lonSupportEMail'};
     my $defdom = &get_domain();
-    my ($to,$bcc,$addtext) =
-        &Apache::loncommon::build_recipient_list(undef,'helpdeskmail',
-                                                 $defdom,$origmail);
     my $from = $admin;
     my %helpform;
     my %domconfig =
@@ -708,8 +720,12 @@ sub print_request_receipt {
         $coursecode = $env{'form.coursecode'};
         if ($coursecode eq '') {
             my $totcodes = 0;
-            my %coursecodes;
-            $totcodes = &Apache::courseclassifier::retrieve_instcodes(\%coursecodes,$defdom);
+            my $instcats = &Apache::lonnet::get_dom_instcats($defdom);
+            if (ref($instcats) eq 'HASH') {
+                if (ref($instcats->{'codes'}) eq 'HASH') {
+                    $totcodes = scalar(keys(%{$instcats->{'codes'}}));
+                }
+            }
             my @standardnames = &Apache::loncommon::get_standard_codeitems();
             if ($totcodes > 0) {
                 my $noregexps = 1;
@@ -878,6 +894,33 @@ $lt{'date'}: $reporttime
             }
         }
     }
+
+    my ($requname,$requdom,$reqemail);
+    foreach my $field ('uname','udom','email') {
+        $env{'form.'.$field} =~ s/^\s+//;
+        $env{'form.'.$field} =~ s/\s+$//;
+    }
+    if ($env{'form.uname'} =~ /^$match_username$/) {
+        $requname = $env{'form.uname'};
+    }
+    if ($env{'form.udom'} =~ /^$match_domain$/) {
+        $requdom = $env{'form.udom'};
+    }
+    if ($env{'form.email'} =~ m/^[^\@]+\@[^\@]+$/) {
+        $reqemail = $env{'form.email'};
+    }
+
+    my $dom_in_effect;
+    unless ($env{'user.domain'} eq 'public') {
+        $dom_in_effect = $env{'user.domain'};
+    }
+    if ($dom_in_effect eq '') {
+        $dom_in_effect = $requdom;
+    }
+    if ($dom_in_effect eq '') {
+        $dom_in_effect = $defdom;
+    }
+
     $displaymsg .= '<span class="LC_helpform_receipt_cat">'.
                    $lt{'date'}.'</span>: '.$reporttime.'<br />'."\n";
 
@@ -900,6 +943,11 @@ END
         &print_header($r,$url,'process');
     }
     my $bad_email = 0;
+    my ($to,$bcc,$addtext) =
+        &Apache::loncommon::build_recipient_list(undef,'helpdeskmail',
+                                                 $dom_in_effect,$origmail,
+                                                 $requname,$requdom,
+                                                 $reqemail);
     if ($to =~ /,/) {
         my @ok_email; 
         foreach my $email (split(/,/,$to)) {
@@ -937,12 +985,8 @@ END
     }
     $r->print(&Apache::loncommon::confirmwrapper($message));
 
-    if (defined($env{'form.email'})) {
-        $env{'form.email'} =~ s/^\s+//;
-        $env{'form.email'} =~ s/\s+$//;
-        if ($env{'form.email'} =~ m/^[^\@]+\@[^\@]+$/) {
-            $from = $env{'form.email'};
-        }
+    if ($reqemail ne '') {
+        $from = $reqemail;
     }
 
     if (defined($env{'form.cc'})) {
@@ -991,7 +1035,9 @@ END
     if ($$cookie{'lonID'} =~ /lonID=($LONCAPA::handle_re);/) {
         $cookies{'lonID'} = $1;
     }
-
+    if ($$cookie{'lonLinkID'} =~ /lonLinkID=([a-f0-9]+_linked);/) {
+        $cookies{'lonLinkID'} = $1;
+    }
     if ($attachmentpath =~ m-/([^/]+)$-) {
         $fname = $1;
         $displaymsg .= '<br />'
@@ -1006,6 +1052,7 @@ END
         foreach my $var(@ENVvars) {
             $supportmsg .= "$var: $ENV{$var}\n";
         }
+        $supportmsg .= "REMOTE_ADDR: $ip\n";
         foreach my $var (@envvars) {
             $supportmsg .= "$var: $env{$var}\n";
         }
@@ -1026,6 +1073,7 @@ END
         foreach my $var (@ENVvars) {
             $attachment_text .= "$var: $ENV{$var}\n";
         }
+        $attachment_text .= "REMOTE_ADDR: $ip\n";
         foreach my $var (@envvars) {
             $attachment_text .= "$var: $env{$var}\n";
         }
@@ -1100,6 +1148,10 @@ END
                        $var.'</span>:&nbsp;'.$ENV{$var}.', ';
         }
     }
+    if ($ip ne '') {
+        $envmsg .= '<span class="LC_helpform_receipt_cat">'.
+                   'REMOTE_ADDR</span>:&nbsp;'.$ip.', ';
+    }
     foreach my $var (@envvars) {
         if ($env{$var} ne '') { 
             $envmsg .= '<span class="LC_helpform_receipt_cat">'.
