@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 # CGI-script to run LaTeX, dvips, ps2ps, ps2pdf etc.
 #
-# $Id: printout.pl,v 1.164 2017/01/29 23:53:55 raeburn Exp $
+# $Id: printout.pl,v 1.168 2021/01/25 18:06:55 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -41,6 +41,7 @@ use Apache::lonmsg();
 use LONCAPA::Enrollment;
 use LONCAPA::Configuration;
 use LONCAPA;
+use Archive::Zip qw( :ERROR_CODES );
 
 use strict;
 
@@ -210,6 +211,9 @@ if ($student_names=~/_END_/) {
     @names_pack=split(/_ENDPERSON_/,$student_names);
 }
 if ($backref) {
+    if ($backref =~ m{^(/uploaded/$LONCAPA::match_domain/$LONCAPA::match_courseid/default_\d+.page)}) {
+        $backref = $1;
+    }
     print('<p>'.&mt("[_1]Return[_2] to resource.",
 		    "<a href=\"$backref\"><b>","</b></a>").'</p>');
     print('<p><a href="javascript:gopost(\'/adm/printout\',\''.$backref.'\');">'.
@@ -597,26 +601,46 @@ foreach $texfile (@texfile) {
          .'</p>';
   }
 }
+if ($advanced_role) { &Apache::lonhtmlcommon::Close_PrgWin('',\%prog_state); }
 print "<br />";
 if ($number_of_files>1) {
+    print('<p>'.&mt('Zip Output:')."\n");
+    my %zip_prog_state;
+    if ($advanced_role) { %zip_prog_state=&Apache::lonhtmlcommon::Create_PrgWin('',$number_of_files); }
     my $zipfile=$texfile[0];
     $zipfile=~s/\.tex/\.zip/;
-    my $statement="zip $zipfile";
+    my $zip = Archive::Zip->new();
+    my $counter = 0;
     foreach my $file (@texfile) {
-	$file=~s/\.tex/.\pdf/;
-	$statement.=' '.$file; 
+        $file=~s/\.tex/.\pdf/;
+        my $dest=$file;
+        $dest=~s{^\Q$perlvar{'lonPrtDir'}\E}{prtspool};
+        $zip->addFile($file,$dest);
+        $dest=~s/^prtspool//;
+        $counter ++;
+        if ($advanced_role) {
+            &Apache::lonhtmlcommon::Update_PrgWin('',\%zip_prog_state,
+                                                  &mt('[_1] added to zip archive ([_2] of [_3]',
+                                                  $dest,$counter,$number_of_files));
+        }
     }
-    print('<p>'.&mt('Zip Output:')."\n<pre>\n");
-    system($statement);
-    print("</pre></p>\n");
-    $zipfile=~s{^\Q$perlvar{'lonPrtDir'}\E}{/prtspool};
-    print
-        '<p>'
-       .&mt('A [_1]ZIP file[_2] of all the PDF files is ready for download.',
-            '<a href="'.$zipfile.'">','</a>')
-       .'</p>';
+    if ($advanced_role) {
+        &Apache::lonhtmlcommon::Update_PrgWin('',\%zip_prog_state,&mt('Writing zip file'));
+    }
+    if ($zip->writeToFileNamed($zipfile) == AZ_OK) {
+        $zipfile=~s{^\Q$perlvar{'lonPrtDir'}\E}{/prtspool};
+        print
+            '<p>'
+           .&mt('A [_1]ZIP file[_2] of all the PDF files is ready for download.',
+                '<a href="'.$zipfile.'">','</a>')
+           .'</p>';
+    } else {
+        print '<p class="LC_error">'.
+              &mt('An error occurred creating a ZIP file of all the PDF files').
+             '</p>';
+    }
+    if ($advanced_role) { &Apache::lonhtmlcommon::Close_PrgWin('',\%zip_prog_state); }
 }
-if ($advanced_role) { &Apache::lonhtmlcommon::Close_PrgWin('',\%prog_state); }
 print(&Apache::loncommon::end_page());
 my $done;
 
@@ -960,7 +984,6 @@ sub convert_figure {
 	&File::Path::mkpath($path,0,0777);
 	$not_eps =~ s/^\s+//;
 	$not_eps =~ s/\s+$//;
-	$not_eps =~ s/ /\\ /g;
     my $prettyname=$not_eps;
 	if ($advanced_role) {
 	    $prettyname=~s|$perlvar{'lonDocRoot'}/|/|;
@@ -980,18 +1003,29 @@ sub convert_figure {
 	    $eps_f = $perlvar{'lonPrtDir'}.'/'.$eps_f;
 
 	    &debug("Converting pdf $not_eps to postscript: $eps_f");
-	    system("pdftops $not_eps $eps_f");
-	    $pdfs_converted++;	# Need to fix ps in last pass.
-	} else {
-	    system("convert $not_eps $eps_f");
-        if($? and $advanced_role){
-            print '<p class="LC_warning">'
-                 .&mt('An error occurred during the conversion of [_1].',
+            my @args = ('pdftops',$not_eps,$eps_f);
+            system({$args[0]} @args); # Indirect object forces list processing mode.
+                                      # See perlfunc documentation for exec().
+            if ($? and $advanced_role) {
+                print '<p class="LC_warning">'
+                     .&mt('An error occurred during the conversion of [_1] to postscript.',
                           '<span class="LC_filename">'.$prettyname.'</span>')
-                 .'<br />'
-                 .&mt('If possible try to save this image using different settings and republish it.')
-                 .'</p>';
-        }
+                     .'</p>';
+            } else {
+                $pdfs_converted++; # Need to fix ps in last pass.
+            }
+	} else {
+            my @args = ('convert',$not_eps,$eps_f);
+            system({$args[0]} @args); # Indirect object forces list processing mode.
+                                      # See perlfunc documentation for exec().
+            if ($? and $advanced_role) {
+                print '<p class="LC_warning">'
+                     .&mt('An error occurred during the conversion of [_1].',
+                          '<span class="LC_filename">'.$prettyname.'</span>')
+                     .'<br />'
+                     .&mt('If possible try to save this image using different settings and republish it.')
+                     .'</p>';
+            }
 	}
 
 	if (not -e $eps_f) {
