@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # Allow users to self-enroll in a course
 #
-# $Id: selfenroll.pm,v 1.27.2.5 2016/08/07 01:40:55 raeburn Exp $
+# $Id: selfenroll.pm,v 1.27.2.6 2020/01/09 17:36:09 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -665,9 +665,10 @@ sub pending_selfenrollment_form {
                 if ($buttontext eq '') {
                     $buttontext = &mt('Complete my enrollment');
                 }
+                my $hostname = &Apache::lonnet::hostname($lonhost);
                 my $protocol = $Apache::lonnet::protocol{$lonhost};
                 $protocol = 'http' if ($protocol ne 'https');
-                my $enroller = $protocol.'://'.&Apache::lonnet::hostname($lonhost).'/cgi-bin/enrollqueued.pl';
+                my $enroller = $protocol.'://'.$hostname.'/cgi-bin/enrollqueued.pl';
                 $output .= '<input type="hidden" name="enroller" value="'.$enroller.'" />'."\n".
                            '<input type="hidden" name="token" value="'.$token.'" />'."\n".
                            '<input type="submit" name="validate" value="'.$buttontext.'" />'."\n".
