@@ -1,7 +1,7 @@
 # The LearningOnline Network
 # URL translation for User Files
 #
-# $Id: lontrans.pm,v 1.14 2011/10/21 16:03:06 www Exp $
+# $Id: lontrans.pm,v 1.14.10.1 2020/03/05 22:02:32 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -52,8 +52,13 @@ sub handler {
 	    $r->filename(&propath($udom,$uname).
 			 '/userfiles/'.(join('/',@ufile)));
         }
-    } else { return DECLINED; }
-    return OK;
+        return OK;
+    } elsif ($r->uri =~ m{^\Q/adm/wrapper/ext/https:/\E[^/]}) {
+        my $uri = $r->uri;
+        $uri =~ s{^(\Q/adm/wrapper/ext/https:/\E)}{$1/};
+        $r->uri($uri);
+    }
+    return DECLINED;
 }
 
 1;
