@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 # The LearningOnline Network
 #
-# $Id: refresh_courseids_db.pl,v 1.20 2016/01/27 22:23:09 raeburn Exp $
+# $Id: refresh_courseids_db.pl,v 1.22 2020/04/22 16:43:56 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -224,11 +224,13 @@ sub recurse_courses {
                 if (($chome ne '')  && ($lastaccess->{$cid} > $twodaysago)) {
                     $env{'request.course.id'} = $cdom.'_'.$cnum;
                     $env{'request.role'} = 'cc./'.$cdom.'/'.$cnum;
+                    $env{'request.role.adv'} = 1;
 
                     my $readmap = 1;
                     ($reqdmajor,$reqdminor) = &Apache::lonrelrequtils::get_release_req($cnum,$cdom,
                                                                                        $crstype,$readmap,
                                                                                        $globals_set);
+                    delete($env{'request.role.adv'});
                     delete($env{'request.course.id'});
                     delete($env{'request.role'});
                 } elsif ($releaserequired) {
@@ -333,7 +335,6 @@ sub recurse_courses {
                             $changes{'internal.releaserequired'} = $releasereq;
                         } else {
                             if ($courseinfo{'internal.releaserequired'} ne $releasereq) {
-                        
                                 $changes{'internal.releaserequired'} = $releasereq;
                             }
                         }
