@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Page Handler
 #
-# $Id: lonpage.pm,v 1.111.2.4 2016/10/29 13:47:21 raeburn Exp $
+# $Id: lonpage.pm,v 1.111.2.13 2020/11/11 21:52:05 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -41,7 +41,9 @@ use Apache::lonxml();
 use Apache::lonlocal;
 use Apache::lonmenu;
 use Apache::lonhomework;
+use Apache::lonenc();
 use HTML::TokeParser;
+use HTML::Entities();
 use GDBM_File;
 use Apache::lonsequence;
 use lib '/home/httpd/lib/perl/';
@@ -184,6 +186,9 @@ sub handler {
       my $fn=$env{'request.course.fn'};
       if (-e "$fn.db") {
           my %buttonshide;
+          my $hostname = $r->hostname();
+          my $lonhost = $r->dir_config('lonHostID');
+          my $ip = &Apache::lonnet::get_host_ip($lonhost);
           if (tie(%hash,'GDBM_File',"$fn.db",&GDBM_READER(),0640)) {
 # ------------------------------------------------------------------- Hash tied
               my $firstres=$hash{'map_start_'.$requrl};
@@ -241,6 +246,8 @@ sub handler {
                   my %ssilink=();
                   my %ssivlink=();
                   my %ssialink=();
+                  my %cssrefs=();
+                  my %httpref=();
      
                   my %cellemb=();
                   my %cellexternal=();
@@ -262,28 +269,79 @@ sub handler {
                       foreach (@colcont) {
                           my $src=$hash{'src_'.$_};
                           my $plainsrc = $src;
+                          my $anchor;
+                          if ($hash{'ext_'.$_} eq 'true:') {
+                              $cellexternal{$_}=($hash{'ext_'.$_} eq 'true:');
+                              $src =~ s{^/ext/}{http://};
+                              $src =~ s{http://https://}{https://};
+                              if ($src =~ /(\#[^#]+)$/) {
+                                  $anchor = $1;
+                                  $src =~ s/\#[^#]+$//;
+                              }
+                          }
+                          my $unencsrc = $src;
                           my ($extension)=($src=~/\.(\w+)$/);
-			  $cellexternal{$_}=($hash{'ext_'.$_} eq 'true:');
 			  if ($hash{'encrypted_'.$_}) {
 			      $src=&Apache::lonenc::encrypted($src);
 			  }
                           my ($mapid,$resid)=split(/\./,$_);
-                          my $symb=&Apache::lonnet::encode_symb($hash{'map_id_'.$mapid},$resid,$src);
+                          my $symb=&Apache::lonnet::encode_symb($hash{'map_id_'.$mapid},$resid,$plainsrc);
                           unless ($env{'request.role.adv'}) {
                               $buttonshide{$symb} = &Apache::lonnet::EXT("resource.0.buttonshide",$symb);
                           }
                           $cellemb{$_}=
 			      &Apache::loncommon::fileembstyle($extension);
                           if ($cellexternal{$_}) {
-                              unless (($target eq 'tex') || ($target eq 'tex_answer')) {
-                                  $ssibody{$_} = <<ENDEXT;
-<iframe src="$src" width="100%">No iframe support!</iframe>
+                              if (($target eq 'tex') || ($target eq 'tex_answer')) {
+                                  my $shown = $src.$anchor;
+                                  if (($hash{'encrypted_'.$_}) && (!$env{'request.role.adv'})) {
+                                      $shown = &mt('URL not shown (encrypted)');
+                                  }
+                                  my $title=&Apache::lonnet::gettitle($symb);
+                                  $title = &Apache::lonxml::latex_special_symbols($title);
+                                  $shown = &Apache::lonxml::latex_special_symbols($shown);
+                                  $ssibody{$_} = ' \strut \\\\ \textit{'.$title.'} \strut \\\\ '.$shown.'\\\\';
+                              } else {
+                                  my $showsrc = $src;
+                                  my ($is_pdf,$title,$linktext);
+                                  if ($unencsrc =~ /\.pdf$/i) {
+                                      $is_pdf = 1;
+                                  }
+                                  if (($hash{'encrypted_'.$_}) && ($symb)) {
+                                      $title=&Apache::lonnet::gettitle(&Apache::lonenc::encrypted($symb));
+                                  } else {
+                                      $title=&Apache::lonnet::gettitle($symb);
+                                  }
+                                  if ($env{'browser.mobile'}) {
+                                      if ($is_pdf) {
+                                          $linktext = &mt('Link to PDF (for mobile devices)');
+                                          $ssibody{$_} = &create_extlink($unencsrc,$anchor,$title,$linktext);
+                                      } else {
+                                          $linktext = &mt('Link to resource');
+                                          $ssibody{$_} = &create_extlink($unencsrc,$anchor,$title,$linktext);
+                                      }
+                                  } else {
+                                      my $absolute = $env{'request.use_absolute'};
+                                      my $uselink = &Apache::loncommon::is_nonframeable($unencsrc,$absolute,$hostname,$ip);
+                                      if (($uselink) || (($ENV{'SERVER_PORT'} == 443) && ($unencsrc =~ m{^http://}))) {
+                                          $linktext = &mt('Link to resource');
+                                          $ssibody{$_} =  &create_extlink($unencsrc,$anchor,$title,$linktext);
+                                      } else {
+                                          if (($hash{'encrypted_'.$_}) && ($symb) && (!$env{'request.role.adv'})) {
+                                              $showsrc .= '?symb='.&Apache::lonenc::encrypted($symb);
+                                          } elsif ($anchor) {
+                                              $showsrc .= $anchor;
+                                          }
+                                          $ssibody{$_} = <<ENDEXT;
+<iframe src="$showsrc" width="100%" height="300px">No iframe support!</iframe>
 ENDEXT
+                                      }
+                                  }
                               }
                           } elsif ($cellemb{$_} eq 'ssi') {
 # --------------------------------------------------------- This is an SSI cell
-			      my $prefix=$_.'_';
-                              my $idprefix= join('_',($mapid,$resid,''));
+			      my $prefix='p_'.$_.'_';
+                              my $idprefix='p_'.join('_',($mapid,$resid,''));
                               my %posthash=('request.prefix' => $prefix,
 					    'LONCAPA_INTERNAL_no_discussion' => 'true',
 					    'symb' => $symb);
@@ -296,26 +354,31 @@ ENDEXT
 				  $posthash{'rndseed'}=$env{'form.rndseed'};
                                   $posthash{'answer_output_mode'} = $env{'form.answer_output_mode'};
 			      }
-			      my $submitted=exists($env{'form.all_submit'});
+			      my $submitted=$env{'form.all_submit_pressed'};
 			      if (!$submitted) {
 				  foreach my $key (keys(%env)) {
-				      if ($key=~/^form.\Q$prefix\Esubmit_/) {
-					  $submitted=1;last;
+				      if ($key=~/^\Qform.$prefix\Esubmit_(.+)_pressed$/) {
+                                          if ($env{$key}) {
+                                              $submitted=1;
+                                              last;
+                                          }
 				      }
-				  }
+                                  }
 			      }
                               if ($submitted) {
 				  foreach my $key (keys(%env)) {
-				      if ($key=~/^form.\Q$prefix\E/) {
+				      if ($key=~/^\Qform.$prefix\E/) {
 					  my $name=$key;
-					  $name=~s/^form.\Q$prefix\E//;
+					  $name=~s/^\Qform.$prefix\E//;
 					  $posthash{$name}=$env{$key};
-				      }
+                                      }
 				  }
-				  if (exists($env{'form.all_submit'})) {
+				  if ($env{'form.all_submit_pressed'}) {
 				      $posthash{'all_submit'}='yes';
 				  }
-			      }
+			      } elsif ($env{'form.'.$prefix.'markaccess'} eq 'yes') {
+		                  $posthash{'markaccess'} = $env{'form.'.$prefix.'markaccess'};
+		              }
                               if ($env{'environment.remote'} eq 'on') {
                                   $posthash{'inhibitmenu'} = 'yes';
                               }
@@ -340,6 +403,7 @@ ENDEXT
                               my $bodydef=0;
                               my $thisxml=0;
                               my @rlinks=();
+                              my @css_hrefs=();
                               if ($output=~/\?xml/) {
                                  $isxml=1;
                                  $thisxml=1;
@@ -382,6 +446,14 @@ ENDEXT
                                            ($bodydef==0)) {
 				      $allscript.="\n\n"
                                                 .$parser->get_text('/script');
+                                  } elsif (($token->[1] eq 'link') &&
+                                           ($bodydef==0)) {
+                                      if (($token->[2]->{'href'} !~ m{^/adm/}) &&
+                                          ($token->[2]->{'rel'} eq 'stylesheet')) {
+                                              $css_hrefs[$#css_hrefs+1]=
+                                                  $token->[2]->{'href'};
+
+                                      }
                                   }
 			        }
 			      }
@@ -405,6 +477,8 @@ ENDEXT
 				      s/\<((?:input|select|button|textarea)[^\>]+)name\s*\=\s*[\'\"]*([^\'\"]+)[\'\"]*([^\>]*)\>/\<$1 name="$prefix$2" $3\>/gsi;
                                   $output=~
                                       s/\<((?:input|select|button|textarea)[^\>]+)id\s*\=\s*[\'\"]*([^\'\"]+)[\'\"]*([^\>]*)\>/\<$1 id="$idprefix$2" $3\>/gsi;
+                                  $output=~
+                                      s/(\Qthis.form.elements['\E)(HW(?:VAL|CHK)_[^']+\'\]\.(?:value=\'|checked))/$1$prefix$2/gsi;
                                   if ($hastimer) {
                                       $output=~
                                           s/\<(input[^\>]+name=\Q"$prefix\Eaccessbutton"[^\>]+)(?:\Qdocument.markaccess.submit();\E)([^\>]*)\>/\<$1pageTimer(this.form,'$prefix')$2\>/gsi;
@@ -424,8 +498,6 @@ ENDEXT
                                   $output=~
                                       s/(\Q<td class="LC_status_\E)(\Qsubmit_\E)([^\"]*)(\s*[^\"]*"\>)/$1$idprefix$2$3$4/g;
                                   if ($nuploads) {
-                                      $output=~
-                                          s/\<(input[^\>]+name=\"\Q$prefix\EHWFILE[^\>]+)\s*id\s*\=\s*[\'\"]*([^\'\"]+)[\'\"]*([^\)]*)\>/\<$1 id="$prefix$2" $3\>/gsi;
                                        ($turninpaths{$prefix},$multiresps{$prefix}) = 
                                            &Apache::loncommon::get_turnedin_filepath($symb,$env{'user.name'},$env{'user.domain'});
                                        if ($turninparent eq '') {
@@ -503,6 +575,28 @@ ENDEXT
                      $output=~s/(\"|\'|\=\s*)$_(\"|\'|\s|\>)/$1$newlocation$2/;
 				  }
 			      }
+                              foreach my $css_href (@css_hrefs) {
+                                  next if ($css_href eq '');
+                                  unless ($css_href =~ m{https?://}) {
+                                      my $proburl = &Apache::lonnet::clutter($plainsrc);
+                                      unless ($css_href =~ m{^/}) {
+                                          my $probdir = $proburl;
+                                          $probdir=~s/\/[^\/]*$//;
+                                          $css_href = &Apache::lonnet::hreflocation($probdir,$css_href);
+                                      }
+                                      if ($css_href =~ m{^/(res|uploaded)/}) {
+                                          unless (($env{'httpref.'.$css_href}) ||
+                                                  ($httpref{'httpref.'.$css_href}) ||
+                                                  (&Apache::lonnet::is_on_map($css_href))) {
+                                              if ($env{'httpref.'.$proburl}) {
+                                                  $proburl = $env{'httpref.'.$proburl};
+                                              }
+                                              $httpref{'httpref.'.$css_href} = $proburl;
+                                          }
+                                      }
+                                  }
+                                  $cssrefs{$css_href} = 1;
+                              }
 # -------------------------------------------------- Deal with Applet codebases
   $output=~s/(\<applet[^\>]+)(codebase\=[^\S\>]+)*([^\>]*)\>/$1.($2?$2:' codebase="'.$thisdir.'"').$3.'>'/gei;
 			      $ssibody{$_}=$output;
@@ -538,7 +632,9 @@ ENDEXT
 				  "\n</script>\n";
 			  }
                           if (($nforms) && ($nuploads)) {
-                              $allscript .= &Apache::lonhtmlcommon::file_submissionchk_js(\%turninpaths,\%multiresps);
+                              $allscript .= &Apache::lonhtmlcommon::file_submissionchk_js(\%turninpaths,\%multiresps).
+                                            '<script type="text/javascript" '.
+                                            'src="/res/adm/includes/file_upload.js"></script>';
                           }
                           if (($nforms) && (&Apache::lonhtmlcommon::htmlareabrowser())) {
                               my %textarea_args = (
@@ -556,6 +652,7 @@ ENDEXT
                                             '// ]]>'.
                                             "\n</script>\n";
                           }
+                          &Apache::lonhtmlcommon::clear_breadcrumb_tools();
                           if (keys(%hastimeleft)) {
                               my (%uniquetimes,%uniquedisplays);
                               foreach my $item (values(%hastimeleft)) {
@@ -599,6 +696,21 @@ ENDEXT
                                   }
                               }
                           }
+                          if (keys(%cssrefs)) {
+                              my $links;
+                              if (keys(%cssrefs)) {
+                                  foreach my $css_href (keys(%cssrefs)) {
+                                      next unless ($css_href =~ m{^(/res/|/uploaded/|https?://)});
+                                      $links .= '<link rel="stylesheet" type="text/css" href="'.$css_href.'" />'."\n";
+                                  }
+                              }
+                              if ($links) {
+                                  if (keys(%httpref)) {
+                                      &Apache::lonnet::appenv(\%httpref);
+                                  }
+                                  $allscript .= "\n$links";
+                              }
+                          }
 # ------------------------------------------------------------------ Start body
 			  $r->print(&Apache::loncommon::start_page(undef,$allscript,
 								   {'force_register' => 1,
@@ -649,7 +761,7 @@ ENDEXT
                           my $avespan=$lcm/($#colcont+1);
                           for ($j=0;$j<=$#colcont;$j++) {
                               my $rid=$colcont[$j];
-			      my $metainfo =&get_buttons(\%hash,$rid,\%buttonshide).'<br />';
+			      my $metainfo =&get_buttons(\%hash,$rid,\%buttonshide,$hostname).'<br />';
 			    unless (($target eq 'tex') || ($target eq 'tex_answer')) {
 				$r->print('<td colspan="'.$avespan.'"');
 			    }
@@ -675,10 +787,7 @@ ENDEXT
 				      }             
 				      $r->print('>');
 				  }
-                                  unless (($cellexternal{$rid}) && 
-                                          ($target eq 'tex') && ($target eq 'tex_answer')) {
-                                      $r->print($ssibody{$rid});
-                                  }
+                                  $r->print($ssibody{$rid});
 				  unless (($target eq 'tex') || ($target eq 'tex_answer')) {
 				      $r->print('</font>');
                                   }
@@ -739,6 +848,8 @@ ENDEXT
                           $r->print(
 	                  '<input name="all_submit" value="'.&mt('Submit All').'" type="'.
 			  (($nforms>1)?'submit':'hidden').'"'.$class.' id="all_submit" />'.
+                          '<input type="hidden" name="all_submit_pressed" '.
+                          'id="all_submit_pressed" value="" />'.
                           '<div id="msg_all_submit" style="display:none">'.
                           &mt('Processing your submission ...').'</div></form>');
                       }
@@ -777,37 +888,50 @@ ENDEXT
 }
 
 sub get_buttons {
-    my ($hash,$rid,$buttonshide) = @_;
+    my ($hash,$rid,$buttonshide,$hostname) = @_;
 
-    my $metainfo = '';
-    my $esrc=&Apache::lonnet::declutter($hash->{'src_'.$rid});
     my ($mapid,$resid)=split(/\./,$rid);
     my $symb=&Apache::lonnet::encode_symb($hash->{'map_id_'.$mapid},
 					  $resid,
 					  $hash->{'src_'.$rid});
+    my $aname;
+    if (($hash->{'encrypted_'.$rid}) && (!$env{'request.role.adv'})) {
+        $aname = 'LC_'.$rid;
+    } else {
+        my $shownsymb = $symb;
+        if ($symb =~ /\#([^\#]+)$/) {
+            my $escan = &escape('#');
+            $shownsymb =~ s/#([^\#]+)$/$escan$1/;
+        }
+        $aname = &escape($shownsymb);
+    }
+    my $metainfo = '<a name="'.$aname.'"></a>';
     unless ($env{'request.role.adv'}) {
         if ($buttonshide->{$symb} eq 'yes') {
-            return;
+            return $metainfo;
         }
     }
+    my $crs_sec = $env{'request.course.id'} . (($env{'request.course.sec'} ne '')
+                                               ? "/$env{'request.course.sec'}"
+                                               : '');
+    my $esrc=&Apache::lonnet::declutter($hash->{'src_'.$rid});
     if ($hash->{'encrypted_'.$rid}) {
-	$symb=&Apache::lonenc::encrypted($symb);
 	$esrc=&Apache::lonenc::encrypted($esrc);
     }
     if ($hash->{'src_'.$rid} !~ m-^/uploaded/-
-        && $hash->{'src_'.$rid} !~ m{^https?://}
+        && $hash->{'src_'.$rid} !~ m{^/ext/}
 	&& !$env{'request.enc'}
 	&& ($env{'request.role.adv'}
 	    || !$hash->{'encrypted_'.$rid})) { 
-	$metainfo .='<a name="'.&escape($symb).'" />'.
-	    '<a href="'.$hash->{'src_'.$rid}.'.meta'.'" target="LONcatInfo">'.
+	$metainfo .= '<a href="'.$hash->{'src_'.$rid}.'.meta'.'" '.
+	    'target="LONcatInfo">'.
             '<img src="/res/adm/pages/catalog.png" class="LC_icon"'.
             ' alt="'.&mt('Show Metadata').'"'.
             ' title="'.&mt('Show Metadata').'" />'.
 	    '</a>';
     }
     if (($hash->{'src_'.$rid} !~ m{^/uploaded/}) &&
-        ($hash->{'src_'.$rid} !~ m{^https?://})) {
+        ($hash->{'src_'.$rid} !~ m{^/ext/})) {
         $metainfo .= '<a href="/adm/evaluate?postdata='.
 	    &escape($esrc).
 	    '" target="LONcatInfo">'.
@@ -819,7 +943,8 @@ sub get_buttons {
     if (($hash->{'src_'.$rid}=~/$LONCAPA::assess_re/) &&
 	($hash->{'src_'.$rid} !~ m-^/uploaded/-)) {
 
-	if (&Apache::lonnet::allowed('mgr',$env{'request.course.id'})) {
+	if ((&Apache::lonnet::allowed('mgr',$crs_sec)) ||
+            (&Apache::lonnet::allowed('vgr',$crs_sec))) {
 	    $metainfo.=
 		'<a href="/adm/grades?symb='.&escape($symb).
 #               '&command=submission" target="LONcatInfo">'.
@@ -827,7 +952,10 @@ sub get_buttons {
                 '<img src="/adm/lonMisc/subm_button.png" class="LC_icon"'.
                 ' alt="'.&mt('View Submissions for a Student or a Group of Students').'"'.
                 ' title="'.&mt('View Submissions for a Student or a Group of Students').'" />'.
-		'</a>'.
+		'</a>';
+        }
+        if (&Apache::lonnet::allowed('mgr',$crs_sec)) {
+            $metainfo.=
 		'<a href="/adm/grades?symb='.&escape($symb).
 #               '&command=gradingmenu" target="LONcatInfo">'.
 		'&command=gradingmenu">'.
@@ -836,7 +964,8 @@ sub get_buttons {
                 ' title="'.&mt('Content Grades').'" />'.
 		'</a>';
 	}
-	if (&Apache::lonnet::allowed('opa',$env{'request.course.id'})) {
+	if ((&Apache::lonnet::allowed('opa',$crs_sec)) ||
+            (&Apache::lonnet::allowed('vpa',$crs_sec))) {
 	    $metainfo.=
 		'<a href="/adm/parmset?symb='.&escape($symb).
 #               '" target="LONcatInfo">'.
@@ -847,23 +976,64 @@ sub get_buttons {
 		'</a>';
 	}
     }
-    if (($env{'request.course.id'}) && (&Apache::lonnet::allowed('mdc',$env{'request.course.id'}))) {
+    if ($env{'request.course.id'}) {
         my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
         my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
         my $file=&Apache::lonnet::declutter($hash->{'src_'.$rid});
-        my ($cfile,$home,$switchserver,$forceedit,$forceview) =
-            &Apache::lonnet::can_edit_resource($file,$cnum,$cdom,$hash->{'src_'.$rid},$symb);
-        if ($cfile ne '') {
-            my $jscall = &Apache::lonhtmlcommon::jump_to_editres($cfile,$home,$switchserver,
-                                                                 $forceedit,1,$symb,undef,
-                                                                 &escape($env{'form.title'}));
-            if ($jscall) {
+        my $editbutton = '';
+        if (&Apache::lonnet::allowed('mdc',$env{'request.course.id'})) {
+            my ($cfile,$home,$switchserver,$forceedit,$forceview) =
+                &Apache::lonnet::can_edit_resource($file,$cnum,$cdom,$hash->{'src_'.$rid},$symb);
+            if ($cfile ne '') {
+                my $jscall = &Apache::lonhtmlcommon::jump_to_editres($cfile,$home,$switchserver,
+                                                                     $forceedit,1,$symb,undef,
+                                                                     &escape($env{'form.title'}),
+                                                                     $hostname);
+                if ($jscall) {
+                    $editbutton = 1;
+                    my $icon = 'pcstr.png';
+                    my $label = &mt('Edit');
+                    my $title = &mt('Edit this resource');
+                    my $pic = '<img src="'.&Apache::loncommon::lonhttpdurl('/res/adm/pages/'.$icon).'"'.
+                              ' class="LC_icon" alt="'.$label.'" title="'.$title.'" />';
+                    $metainfo .= '&nbsp;<a href="javascript:'.$jscall.';">'.$pic.'</a>';
+                }
+            }
+        }
+        if ((!$editbutton) && ($file=~/$LONCAPA::assess_re/)) {
+            my $url = &Apache::lonnet::clutter($file);
+            my $viewsrcbutton;
+            if ((&Apache::lonnet::allowed('cre','/')) &&
+                (&Apache::lonnet::metadata($url,'sourceavail') eq 'open')) {
+                $viewsrcbutton = 1;
+            } elsif (&Apache::lonnet::allowed('vxc',$crs_sec)) {
+                if ($url =~ m{^\Q/res/$cdom/\E($LONCAPA::match_username)/}) {
+                    my $auname = $1;
+                    if (($env{'request.course.adhocsrcaccess'} ne '') &&
+                        (grep(/^\Q$auname\E$/,split(/,/,$env{'request.course.adhocsrcaccess'})))) {
+                        $viewsrcbutton = 1;
+                    } elsif ((&Apache::lonnet::metadata($url,'sourceavail') eq 'open') &&
+                             (&Apache::lonnet::allowed('bre',$crs_sec))) {
+                        $viewsrcbutton = 1;
+                    }
+                }
+            }
+            if ($viewsrcbutton) {
                 my $icon = 'pcstr.png';
-                my $label = &mt('Edit');
-                my $title = &mt('Edit this resource');
+                my $label = &mt('View Source');
+                my $title = &mt('View source code');
+                my $jsrid = $rid;
+                $jsrid =~ s/\./_/g;
+                my $showurl = &escape(&Apache::lonenc::check_encrypt($url));
                 my $pic = '<img src="'.&Apache::loncommon::lonhttpdurl('/res/adm/pages/'.$icon).'"'.
                           ' class="LC_icon" alt="'.$label.'" title="'.$title.'" />';
-                $metainfo .= '&nbsp;<a href="javascript:'.$jscall.';">'.$pic.'</a>';
+                $metainfo .= '&nbsp;<a href="javascript:open_source_'.$jsrid.'();">'.$pic.'</a>'."\n".
+                             '<script type="text/javascript">'."\n".
+                             "function open_source_$jsrid() {\n".
+                             "  sourcewin=window.open('/adm/source?inhibitmenu=yes&viewonly=1&filename=$showurl','LONsource',".
+                             "'height=500,width=600,resizable=yes,location=no,menubar=no,toolbar=no,scrollbars=yes');\n".
+                             "}\n".
+                             "</script>\n";
             }
         }
     }
@@ -893,11 +1063,20 @@ $collapse
 <span id="ddcountexpand" class="LC_menubuttons_inline_text" >$expand</span>
 <img src="/res/adm/pages/timer.png" title="$desc" class="LC_icon" alt="$alttxt" /><span class="LC_menubuttons_inline_text">$title</span></a>
 END
-    &Apache::lonhtmlcommon::clear_breadcrumb_tools();
     &Apache::lonhtmlcommon::add_breadcrumb_tool('tools',$output);
     return;
 }
 
+sub create_extlink {
+    my ($url,$anchor,$title,$linktext) = @_;
+    my $shownlink;
+    unless ($title eq '') {
+        $shownlink = '<span style="font-weight:bold;">'.$title.'</span><br />';
+    }
+    my $dest = &HTML::Entities::encode($url.$anchor,'&<>"');
+    $shownlink .= '<a href="'.$dest.'">'.$linktext.'</a>';
+    return $shownlink;
+}
 
 1;
 __END__
