@@ -2,7 +2,7 @@
 #
 # Page flip handler
 #
-# $Id: lonpageflip.pm,v 1.80.8.9 2016/08/05 21:48:38 raeburn Exp $
+# $Id: lonpageflip.pm,v 1.80.8.12 2021/03/04 01:33:43 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -255,6 +255,54 @@ sub first_accessible_resource {
     }
 }
 
+sub check_http_req {
+    my ($srcref) = @_;
+    return unless (ref($srcref) eq 'SCALAR');
+    my $usehttp;
+    if ($env{'request.course.id'}) {
+        my $cnum = $env{'course.'.$env{'request.course.id'}.'.num'};
+        my $cdom = $env{'course.'.$env{'request.course.id'}.'.domain'};
+        if (($$srcref =~ m{^\Q/public/$cdom/$cnum/syllabus\E($|\?)}) &&
+            ($ENV{'SERVER_PORT'} == 443) &&
+            ($env{'course.'.$env{'request.course.id'}.'.externalsyllabus'} =~ m{^http://})) {
+            unless (&Apache::lonnet::uses_sts()) {
+                $$srcref .= (($$srcref =~/\?/)? '&':'?') . 'usehttp=1';
+                $usehttp = 1;
+            }
+        } elsif (($$srcref =~ m{^\Q/adm/wrapper/ext/\E(?!https:)}) &&
+                 ($ENV{'SERVER_PORT'} == 443)) {
+            unless (&Apache::lonnet::uses_sts()) {
+                my ($url,$anchor) = ($$srcref =~ /^([^\#]+)(?:|(\#[^\#]+))$/);
+                $$srcref = $url . (($$srcref =~/\?/)? '&':'?') . 'usehttp=1' .$anchor;
+                $usehttp = 1;
+            }
+        }
+    }
+    return $usehttp;
+}
+
+sub reinited_js {
+    my ($url,$cid,$timeout) = @_;
+    if (!$timeout) {
+        $timeout = 0;
+    }
+    return <<"END";
+<script type="text/javascript">
+// <![CDATA[
+setTimeout(function() {
+    var newurl = '$url';
+    if (document.getElementById('LC_update_$cid')) {
+        document.getElementById('LC_update_$cid').style.display = 'none';
+    }
+    if ((newurl !== null) && (newurl !== '') && (newurl !== 'undefined')) {
+        window.location.href = "$url";
+    }
+}, $timeout);
+// ]]>
+</script>
+END
+}
+
 # ================================================================ Main Handler
 
 sub handler {
@@ -271,8 +319,10 @@ sub handler {
   my %cachehash=(); 
   my $multichoice=0;
   my %multichoicehash=();
+  my %prog_state=(); 
   my ($redirecturl,$redirectsymb,$enc,$anchor);
   my $next='';
+  my $hostname = $r->hostname();
   my @possibilities=();
    &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['postdata']);
   if (($env{'form.postdata'})&&($env{'request.course.fn'})) {
@@ -311,8 +361,20 @@ sub handler {
                       untie(%hash);
                   }
               }
+              &Apache::loncommon::content_type($r,'text/html');
+              $r->send_http_header;
+              $r->print(&Apache::loncommon::start_page('Content Changed'));
+              my $preamble = '<div id="LC_update_'.$env{'request.course.id'}.'" class="LC_info">'.
+                             '<br />'.
+                             &mt('Your course session is being updated because of recent changes by course personnel.').
+                             ' '.&mt('Please be patient').'.<br /></div>'.
+                             '<div style="padding:0;clear:both;margin:0;border:0"></div>';
+              %prog_state = &Apache::lonhtmlcommon::Create_PrgWin($r,undef,$preamble);
+              &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,&mt('Updating course'));
               my ($furl,$ferr) = &Apache::lonuserstate::readmap("$cdom/$cnum");
+              &Apache::lonhtmlcommon::Update_PrgWin($r,\%prog_state,&mt('Finished!'));
               if ($ferr) {
+                  &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
                   my $requrl = $r->uri;
                   $env{'user.error.msg'}="$requrl:bre:0:0:Course not initialized";
                   $env{'user.reinit'} = 1;
@@ -329,22 +391,42 @@ sub handler {
       }
       if ($direction eq 'firstres') {
 	  my $furl=&first_accessible_resource();
-	  &Apache::loncommon::content_type($r,'text/html');
-	  $r->header_out(Location => 
-			 &Apache::lonnet::absolute_url().$furl);
-	     
-	  return REDIRECT;
+          my $usehttp = &check_http_req(\$furl);
+          if (($usehttp) && ($hostname ne '')) {
+              $furl='http://'.$hostname.$furl;
+          } else {
+              $furl=&Apache::lonnet::absolute_url().$furl;
+          }
+          if ($reinitcheck eq 'update') {
+              &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
+              $r->print(&reinited_js($furl,$env{'request.course.id'},100));
+              $r->print(&Apache::loncommon::end_page());
+              return OK;
+          } else {
+	      &Apache::loncommon::content_type($r,'text/html');
+	      $r->header_out(Location => $furl);
+	      return REDIRECT;
+          }
       }
-      if ($direction eq 'return') { 
+      if ($direction eq 'return') {
 # -------------------------------------------------------- Return to last known
-         my $newloc;
+         my ($newloc,$usehttp);
          if (($last) && (tie(%hash,'GDBM_File',$env{'request.course.fn'}.'.db',
                         &GDBM_READER(),0640))) {
             my ($murl,$id,$fn)=&Apache::lonnet::decode_symb($last);
 	    $id=$hash{'map_pc_'.&Apache::lonnet::clutter($murl)}.'.'.$id;
 	    $newloc=$hash{'src_'.$id};
 	    if ($newloc) {
-		if ($hash{'encrypted_'.$id}) { $newloc=&Apache::lonenc::encrypted($newloc); }
+                $usehttp = &check_http_req(\$newloc);
+                if ($hash{'encrypted_'.$id}) {
+                    $newloc=&Apache::lonenc::encrypted($newloc);
+                }
+                if ($newloc =~ m{^(/adm/wrapper/ext/[^\#]+)(?:|(\#[^\#]+))$}) {
+                    my ($url,$anchor) = ($1,$2);
+                    if ($anchor) {
+                        $newloc = $url.(($url=~/\?/)?'&':'?').'symb='.&escape($last).$anchor;
+                    }
+                }
 	    } else {
 		$newloc='/adm/navmaps';
 	    }
@@ -352,11 +434,20 @@ sub handler {
          } else {
 	    $newloc='/adm/navmaps';
          }
-	 &Apache::loncommon::content_type($r,'text/html');
-	 $r->header_out(Location => 
-			&Apache::lonnet::absolute_url().$newloc);
-	     
-	 return REDIRECT;
+         if (($usehttp) && ($hostname ne '')) {
+             $newloc='http://'.$hostname.$newloc;
+         } else {
+             $newloc=&Apache::lonnet::absolute_url().$newloc
+         }
+         if ($reinitcheck eq 'update') {
+             $r->print(&reinited_js($newloc,$env{'request.course.id'},100));
+             $r->print(&Apache::loncommon::end_page());
+             return OK;
+         } else {
+             &Apache::loncommon::content_type($r,'text/html');
+             $r->header_out(Location => $newloc);
+             return REDIRECT;
+         }
       }
 #
 # Is the current URL on the map? If not, start with last known URL
@@ -370,15 +461,21 @@ sub handler {
              $last=$hash{'last_known'};
              untie(%hash);
          }
-         my $newloc;
          if ($last) {
 	     $currenturl=&Apache::lonnet::clutter((&Apache::lonnet::decode_symb($last))[2]);
 	 } else {
-	     &Apache::loncommon::content_type($r,'text/html');
-	     $r->header_out(Location => 
-			    &Apache::lonnet::absolute_url().
-			    '/adm/navmaps');
-	     return REDIRECT;
+             my $newloc = &Apache::lonnet::absolute_url().
+                          '/adm/navmaps';
+             if ($reinitcheck eq 'update') {
+                 &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
+                 $r->print(&reinited_js($newloc,$env{'request.course.id'},100));
+                 $r->print(&Apache::loncommon::end_page());
+                 return OK;
+             } else {
+                 &Apache::loncommon::content_type($r,'text/html');
+	         $r->header_out(Location => $newloc);
+	         return REDIRECT;
+             }
          }
       }
 # ------------------------------------------- Do we have any idea where we are?
@@ -459,7 +556,12 @@ sub handler {
 # ------------------------------------- Check for and display critical messages
                   my ($redirect, $url) = &Apache::loncommon::critical_redirect(300);
                   unless ($redirect) {
-                      $url=&Apache::lonnet::absolute_url().$redirecturl;
+                      my $usehttp = &check_http_req(\$redirecturl);
+                      if (($usehttp) && ($hostname ne '')) {
+                          $url='http://'.$hostname.$redirecturl;
+                      } else {
+                          $url=&Apache::lonnet::absolute_url().$redirecturl;
+                      }
                       my $addanchor;
                       if (($anchor ne '') && (!$enc || $env{'request.role.adv'})) {
                           $addanchor = 1;
@@ -470,9 +572,16 @@ sub handler {
                           $url .= $anchor;
                       }
                   }
-                  &Apache::loncommon::content_type($r,'text/html');
-                  $r->header_out(Location => $url);
-                  return REDIRECT;
+                  if ($reinitcheck eq 'update') {
+                      &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
+                      $r->print(&reinited_js($url,$env{'request.course.id'},100));
+                      $r->print(&Apache::loncommon::end_page());
+                      return OK;
+                  } else {
+                      &Apache::loncommon::content_type($r,'text/html');
+                      $r->header_out(Location => $url);
+                      return REDIRECT;
+                  }
 	      } else {
 # --------------------------------------------------------- There was a problem
                   &Apache::loncommon::content_type($r,'text/html');
@@ -507,9 +616,14 @@ $lt{'pick'}:
 <tr><th>$lt{'titleheader'}</th><th>$lt{'type'}</th></tr>
 ENDSTART
                      foreach my $id (@possibilities) {
+                        my $src = $multichoicehash{'src_'.$id};
+                        my $usehttp = &check_http_req(\$src);
+                        if (($usehttp) && ($hostname ne '')) {
+                            $src = 'http://'.$hostname.$src;
+                        }
                         $r->print(
                               '<tr><td><a href="'.
-				  &add_get_param($multichoicehash{'src_'.$id},
+				  &add_get_param($src,
 						 {'symb' =>
 						      $multichoicehash{'symb_'.$id},
 						  }).'">'.
@@ -555,12 +669,23 @@ ENDSTART
 	      }
 	  } else {
 # ------------------------------------------------- Problem, could not tie hash
+              if ($reinitcheck eq 'update') {
+                  &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
+                  $r->print(&Apache::loncommon::end_page());
+              }
               $env{'user.error.msg'}="/adm/flip:bre:0:1:Course Data Missing";
               return HTTP_NOT_ACCEPTABLE; 
           }
       } else {
 # ---------------------------------------- No, could not determine where we are
-	  $r->internal_redirect('/adm/ambiguous');
+          my $newloc = '/adm/ambiguous';
+          if ($reinitcheck eq 'update') {
+              &Apache::lonhtmlcommon::Close_PrgWin($r,\%prog_state);
+              $r->print(&reinited_js($newloc,$env{'request.course.id'},100));
+              $r->print(&Apache::loncommon::end_page());
+          } else {
+	      $r->internal_redirect($newloc);
+          }
           return OK;
       }
   } else {
