@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # Wrapper for external and binary files as standalone resources
 #
-# $Id: lonwrapper.pm,v 1.49.2.4 2017/03/14 04:12:10 raeburn Exp $
+# $Id: lonwrapper.pm,v 1.49.2.10 2020/03/05 22:05:50 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -40,7 +40,7 @@ use Apache::lonextresedit();
 
 # ================================================================ Main Handler
 sub wrapper {
-    my ($url,$brcrum,$absolute,$is_ext,$is_pdf,$title) = @_;
+    my ($r,$url,$brcrum,$absolute,$is_ext,$is_pdf,$title) = @_;
 
     my $forcereg;
     unless ($env{'form.folderpath'}) {
@@ -52,7 +52,7 @@ sub wrapper {
                                           'show' => 'Show content in pop-up window',
                                         );
 
-    my $anchor;
+    my ($anchor,$uselink,$linktext);
     if ($is_ext) {
         if ($env{'form.symb'}) {
             (undef,undef,my $res) = &Apache::lonnet::decode_symb($env{'form.symb'});
@@ -62,6 +62,12 @@ sub wrapper {
         } elsif ($env{'form.anchor'} ne '') {
             $anchor = '#'.$env{'form.anchor'};
         }
+        unless (($is_pdf) && ($env{'browser.mobile'})) {
+            my $hostname = $r->hostname();
+            my $lonhost = $r->dir_config('lonHostID');
+            my $ip = &Apache::lonnet::get_host_ip($lonhost);
+            $uselink = &Apache::loncommon::is_nonframeable($url,$absolute,$hostname,$ip);
+        }
     }
 
     my $noiframe = &Apache::loncommon::modal_link($url.$anchor,$lt{'show'},500,400);
@@ -79,34 +85,61 @@ sub wrapper {
         $args->{'only_body'} = $env{'form.only_body'};
     }
 
-    my $startpage = &Apache::loncommon::start_page('Menu',undef,$args);
+    my $headjs;
+
+#
+# Where iframe is in use, if window.onload() executes before the custom resize function
+# has been defined (jQuery), two global javascript vars (LCnotready and LCresizedef)
+# are used to ensure document.ready() triggers a call to resize, so the iframe contents
+# do not obscure the Functions menu.
+#
+
+    unless (($env{'browser.mobile'}) || $uselink) {
+        $headjs = '
+<script type="text/javascript">
+// <![CDATA[
+var LCnotready = 0;
+var LCresizedef = 0;
+// ]]>
+</script>'."\n";
+    }
+
+    my $startpage = &Apache::loncommon::start_page('Menu',$headjs,$args);
     my $endpage = &Apache::loncommon::end_page();
 
+    if (($uselink) && ($title eq '')) {
+        if ($env{'form.symb'}) {
+            $title=&Apache::lonnet::gettitle($env{'form.symb'});
+        } else {
+            my $symb=&Apache::lonnet::symbread($r->uri);
+            if ($symb) {
+                $title=&Apache::lonnet::gettitle($symb);
+            }
+        }
+    }
     if ($env{'browser.mobile'}) {
         my $output = $startpage;
         if ($is_pdf) {
-            if ($title eq '') {
-                $title = $env{'form.title'};
-                if ($title eq '') {
-                    unless ($env{'request.enc'}) {
-                        ($title) = ($url =~ m{/([^/]+)$});
-                        $title =~ s/(\?[^\?]+)$//;
-                    }
-                }
-            }
-            unless ($title eq '') {
-                $output .= $title.'<br />';
-            }
-            $output .= '<a href="'.$url.'">'.&mt('Link to PDF (for mobile devices)').'</a>';
+            $linktext = &mt('Link to PDF (for mobile devices)');
+            $output .= &create_link($url,$anchor,$title,$linktext);
         } else {
-            $output .= '<div style="overflow:scroll; -webkit-overflow-scrolling:touch;">'."\n".
-                       '<iframe src="'.$url.$anchor.'" height="100%" width="100%" frameborder="0">'."\n".
-                       "$lt{'noif'} $noiframe\n".
-                       "</iframe>\n".
-                       "</div>\n";
+            if ($uselink) {
+                $linktext = &mt('Link to resource');
+                $output .= &create_link($url,$anchor,$title,$linktext);
+            } else {
+                my $dest = &HTML::Entities::encode($url.$anchor,'&<>"');
+                $output .= '<div style="overflow:scroll; -webkit-overflow-scrolling:touch;">'."\n".
+                           '<iframe src="'.$dest.'" height="100%" width="100%" frameborder="0">'."\n".
+                           "$lt{'noif'} $noiframe\n".
+                           "</iframe>\n".
+                           "</div>\n";
+            }
         }
         $output .= $endpage;
         return $output;
+    } elsif ($uselink) {
+        $linktext = &mt('Link to resource');
+        return $startpage.&create_link($url,$anchor,$title,$linktext).$endpage;
     } else {
         my $offset = 5;
         &Apache::loncommon::get_unprocessed_cgi($ENV{'QUERY_STRING'},['inhibitmenu']);
@@ -135,8 +168,20 @@ sub wrapper {
                 var pos = height + hdrtop + offset;
                 \$('.LC_iframecontainer').css('top', pos);
             });
+            LCresizedef = 1;
+            if (LCnotready == 1) {
+                LCnotready = 0;
+                \$(window).trigger('resize');
+            }
         });
-        window.onload = function(){  \$(window).trigger('resize') };
+        window.onload = function(){
+            if (LCresizedef) {
+                LCnotready = 0;
+                \$(window).trigger('resize');
+            } else {
+                LCnotready = 1;
+            }
+        };
 SCRIPT
         # javascript will position the iframe if window was resized (or zoomed)
         return <<ENDFRAME;
@@ -150,6 +195,26 @@ ENDFRAME
     }
 }
 
+sub create_link {
+    my ($url,$anchor,$title,$linktext) = @_;
+    my $shownlink;
+    if ($title eq '') {
+        $title = $env{'form.title'};
+        if ($title eq '') {
+            unless ($env{'request.enc'}) {
+                ($title) = ($url =~ m{/([^/]+)$});
+                $title =~ s/(\?[^\?]+)$//;
+            }
+        }
+    }
+    unless ($title eq '') {
+        $shownlink = '<span style="font-weight:bold;">'.$title.'</span><br />';
+    }
+    my $dest = &HTML::Entities::encode($url.$anchor,'&<>"');
+    $shownlink .= '<a href="'.$dest.'">'.$linktext.'</a>';
+    return $shownlink;
+}
+
 sub handler {
     my $r=shift;
     &Apache::loncommon::content_type($r,'text/html');
@@ -158,13 +223,13 @@ sub handler {
     return OK if $r->header_only;
 
     my $url = $r->uri;
-    my ($is_ext,$brcrum,$absolute,$is_pdf);
+    my ($is_ext,$brcrum,$absolute,$is_pdf,$cdom,$cnum,$hostname);
 
     for ($url){
         s|^/adm/wrapper||;
-        $is_ext = $_ =~ s|^/ext/|http://|;         
-        s|http://https://|https://|;
-        s|&colon;|:|g;              
+        $is_ext = $_ =~ s|^/ext/|http://|;
+        s|http://https://|https://| if ($is_ext);
+        s|&colon;|:|g;
     }
 
     if ($url =~ /\.pdf$/i) {
@@ -188,10 +253,15 @@ sub handler {
                     $url .= '#'.$env{'form.anchor'};
                 }
             }
+            if (($url =~ /^http:/) && ($ENV{'SERVER_PORT'} == 443)) {
+                $hostname = $r->hostname();
+            }
+            my $type = 'ext';
             $r->print(
                 &Apache::lonextresedit::display_editor($url,$env{'form.folderpath'},
                                                        $env{'form.symb'},
-                                                       $env{'form.idx'}));
+                                                       $env{'form.idx'},$type,$cdom,
+                                                       $cnum,$hostname));
             return OK;
         } elsif ($env{'form.folderpath'} =~ /^supplemental/) {
             my $crstype = &Apache::loncommon::course_type();
@@ -230,7 +300,7 @@ sub handler {
         # encrypt url if not external
         &Apache::lonenc::check_encrypt(\$url) if $url !~ /^https?\:/ ;
 
-        $r->print( wrapper($url,$brcrum,$absolute,$is_ext,$is_pdf) );
+        $r->print( wrapper($r,$url,$brcrum,$absolute,$is_ext,$is_pdf) );
 
     } # not just the menu
     
@@ -257,10 +327,14 @@ described at http://www.lon-capa.org.
 
 =over
 
-=item wrapper($url,$brcrum,$absolute,$is_ext,$is_pdf,$title))
+=item wrapper($r,$url,$brcrum,$absolute,$is_ext,$is_pdf,$title))
 
 =over
 
+=item $r
+
+request object
+
 =item $url
 
 url to display by including in an iframe within a
